/*
 * acceleration.c
 *
 *  Created on: 01.01.2011
 *      Author: AndreR
 */

#include "stm32f10x_gpio.h"

#include "acceleration.h"
#include "hal/spi/lis3.h"
#include "commands.h"
#include "network.h"
#include "util/console.h"
#include "spi1_2.h"

#include "FreeRTOS.h"
#include "task.h"

AccelerationGlobal acceleration;

void AccelerationInit()
{
	acceleration.sum.x = 0;
	acceleration.sum.y = 0;
	acceleration.velocity.x = 0;
	acceleration.velocity.y = 0;

	acceleration.logEnabled = 0;

	for(uint8_t i = 0; i < 6; i++)
	{
		acceleration.offset[i] = 0;
		acceleration.raw[i] = 0;
	}

	SPILIS3Init(&acceleration.inner, &spi2, GPIOB, GPIO_Pin_1);	// PIN NOT CONNECTED !!
	SPILIS3Init(&acceleration.outer, &spi1, GPIOB, GPIO_Pin_0);	// PIN NOT CONNECTED !!
}

void AccelerationSetLogging(uint8_t enable)
{
	acceleration.logEnabled = enable;
}

void AccelerationTask(void* params)
{
	params = params;

	portTickType xLastWakeTime = xTaskGetTickCount();
	portTickType xLastLogTime = xLastWakeTime;

	while(1)
	{
		for(uint8_t j = 0; j < 3; j++)
		{
			acceleration.raw[j] = acceleration.inner.raw[j];
		}

		for(uint8_t j = 0; j < 3; j++)
		{
			acceleration.raw[j+3] = acceleration.outer.raw[j];
		}

		for(uint8_t i = 0; i < 6; i++)
		{
			acceleration.raw[i] -= acceleration.offset[i];
		}

		if(xLastWakeTime - xLastLogTime > 50/portTICK_RATE_MS && acceleration.logEnabled)
		{
			xLastLogTime = xLastWakeTime;

			MovementLis3LogRaw log;
			log.iax = acceleration.raw[0];
			log.iay = acceleration.raw[1];
			log.iaz = acceleration.raw[2];
			log.oax = acceleration.raw[3];
			log.oay = acceleration.raw[4];
			log.oaz = acceleration.raw[5];

			PacketHeader header;
			header.section = SECTION_MOVEMENT;
			header.cmd = CMD_MOVEMENT_LIS3_LOG_RAW;
			header.len = LEN_MOVEMENT_LIS3_LOG_RAW;

			NetworkSendPacket(&header, (uint8_t*)&log, 0);
		}

		SPILIS3GetAcceleration(&acceleration.inner);
		SPILIS3GetAcceleration(&acceleration.outer);

		vTaskDelayUntil(&xLastWakeTime, 10/portTICK_RATE_MS);
	}
}

void AccelerationCalibrate()
{
	int32_t offsets[6];

	for(uint8_t i = 0; i < 6; i++)
	{
		offsets[i] = 0;
		acceleration.offset[i] = 0;
	}

	SPILIS3GetAcceleration(&acceleration.inner);
	SPILIS3GetAcceleration(&acceleration.outer);

	vTaskDelay(20/portTICK_RATE_MS);	// wait for new values

	SPILIS3GetAcceleration(&acceleration.inner);
	SPILIS3GetAcceleration(&acceleration.outer);

	vTaskDelay(10/portTICK_RATE_MS);

	for(uint8_t i = 0; i < 10; i++)
	{
		for(uint8_t j = 0; j < 3; j++)
		{
			offsets[j] += acceleration.inner.raw[j];
		}

		for(uint8_t j = 0; j < 3; j++)
		{
			offsets[j+3] += acceleration.outer.raw[j];
		}

		SPILIS3GetAcceleration(&acceleration.inner);
		SPILIS3GetAcceleration(&acceleration.outer);

		vTaskDelay(10/portTICK_RATE_MS);
	}

	for(uint8_t i = 0; i < 6; i++)
	{
		offsets[i] /= 10;
		acceleration.offset[i] = offsets[i];
	}
}
