/*
 * hal.c
 *
 *  Created on: 10.05.2011
 *      Author: AndreR
 */

#include "hal.h"

#define TIMER1_MODE_CHARGE 0
#define TIMER1_MODE_FIRE 1

void setTimer1Mode(uint8_t mode)
{
	if(mode == TIMER1_MODE_CHARGE)
	{
		// init Timer 1 (CHG)
		TCCR1B = (1 << WGM13);
		TCNT1 = 0;
		OCR1A = 0;
		ICR1 = 12;	// determines period (T), in clock cycles (@12Mhz)
		TCCR1A = (1 << COM1A1); // non-inverting Phase and frequency correct PWM, TOP = ICR1
		TIMSK1 = 0;
	}

	if(mode == TIMER1_MODE_FIRE)
	{
		TCCR1B = 0;
		TCCR1A = 0;
		TCNT1 = 0;
		OCR1A = 0;
		OCR1B = 0xFFFF;
		ICR1 = 0xFFFF;
		TIFR1 |= (1 << OCF1B);
		TIMSK1 = (1 << OCIE1B);
	}
}

void SetFanSpeed(uint8_t speed)
{
	if(speed > 150)
		speed = 150;

	OCR2B = speed;
}

void EnableCharge(uint16_t onTicks, uint16_t offTicks)
{
	setTimer1Mode(TIMER1_MODE_CHARGE);

	ICR1 = onTicks + offTicks;
	OCR1A = onTicks;

	TCCR1B |= (1 << CS10);
}

void DisableCharge()
{
	OCR1A = 0;
}

void EnableFire1(uint8_t level, FixedPoint142 duration)
{
	uint8_t prescaler;
	uint16_t value;
	CalcTimer(duration, &prescaler, &value);

	setTimer1Mode(TIMER1_MODE_FIRE);

	OCR1B = value;

	OCR0B = level;
	PORTD &= ~(1 << EN_DIS2);
	PORTD |= (1 << EN_DIS1);

	TCCR1B |= prescaler;
}

void EnableFire2(uint8_t level, FixedPoint142 duration)
{
	uint8_t prescaler;
	uint16_t value;
	CalcTimer(duration, &prescaler, &value);

	setTimer1Mode(TIMER1_MODE_FIRE);

	OCR1B = value;

	OCR0B = level;
	PORTD &= ~(1 << EN_DIS1);
	PORTD |= (1 << EN_DIS2);

	TCCR1B |= prescaler;
}

void DisableFire()
{
	OCR0B = 0;
	PORTD &= ~((1 << EN_DIS1) | (1 << EN_DIS2));
	TIMSK1 &= ~(1 << OCIE1B);
	TCCR1B = 0;
}

void CalcTimer(FixedPoint142 time, uint8_t* pPrescaler, uint16_t* pValue)
{
	uint32_t time32 = time.value;

	if(time.exp == 1)
		time32 *= 10;

	if(time.exp == 2)
		time32 *= 100;

	if(time.exp == 3)
		time32 *= 1000;

	if(time32 < 54610)
	{
		// 0.083us precision
		*pValue = (time32*10000)/8333;
		*pPrescaler = 1;	// 1
	}
	else if(time32 < 436906)
	{
		// 0.666 us precision
		*pValue = (time32*1000)/6667;
		*pPrescaler = 2;	// 8
	}
	else if(time32 < 3495000)
	{
		// 5.333us precision
		*pValue = (time32*100)/5333;
		*pPrescaler = 3;	// 64
	}
	else if(time32 < 13978000)
	{
		// 21.333us
		*pValue = (time32*10)/2133;
		*pPrescaler = 4;	// 256
	}
	else if(time32 < 55920000)
	{
		// 83.333us
		*pValue = (time32*10)/8533;
		*pPrescaler = 5;	// 1024
	}
	else
	{
		*pValue = 1;
		*pPrescaler = 1;
	}
}
