/*
 * buzzer.c
 *
 *  Created on: 23.12.2010
 *      Author: AndreR
 */

#include "buzzer.h"

#include "stm32f10x_gpio.h"
#include "stm32f10x_tim.h"
#include "stm32f10x_rcc.h"
#include "misc.h"

#include "util/bit_banding.h"

#include "constants.h"

#define TIM7_EN		BBPeriphSet(TIM7_BASE, 0)
#define TIM7_DIS	BBPeriphReset(TIM7_BASE, 0)

#define BUZZ (1 << 5)


void TIM7_IRQHandler()
{
	TIM7->SR = 0;

	GPIOC->ODR ^= BUZZ;
}

void BuzzerInit()
{
	// Timer init, TIM7
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM7, ENABLE);

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_Out_PP;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_Pin = BUZZ;
	GPIO_Init(GPIOC, &gpioInit);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 185;	// ~2700Hz
	timeBaseInit.TIM_Prescaler = 72;	// 72 = 1us
	timeBaseInit.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM7, &timeBaseInit);

	TIM_ARRPreloadConfig(TIM7, DISABLE);
	TIM_CCPreloadControl(TIM7, DISABLE);

	TIM_ITConfig(TIM7, TIM_IT_Update, ENABLE);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = TIM7_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_BUZZER;	// lowest priority
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&nvicInit);

	TIM7_DIS;

	GPIOC->BRR = BUZZ;
}

void BuzzerEnable(uint8_t en)
{
	if(en)
	{
		TIM7_EN;
	}
	else
	{
		TIM7_DIS;
		GPIOC->BRR = BUZZ;
	}
}

void BuzzerSetFrequency(uint32_t freq)
{
	TIM7->ARR = 500000/freq;
}
