/*
 * motor.h
 *
 *  Created on: 10.07.2010
 *      Author: AndreR
 */

#ifndef __MOTOR_H__
#define __MOTOR_H__

#include "stm32f10x.h"
#include "stm32f10x_exti.h"

#include "util/bit_banding.h"

/*
 * DEFINES
 */
#define H1Mask ((1 <<  6) | (1 <<  7) | (1 <<  8))
#define H2Mask ((1 <<  6) | (1 <<  7) | (1 <<  8))
#define H3Mask ((1 << 13) | (1 << 14) | (1 << 15))
#define H4Mask ((1 << 13) | (1 << 14) | (1 << 15))
#define H5Mask ((1 <<  6) | (1 <<  7) | (1 <<  8))

#define M1Pos ((GPIOG->IDR & H1Mask) >> 6)
#define M2Pos ((GPIOF->IDR & H2Mask) >> 6)
#define M3Pos ((GPIOE->IDR & H3Mask) >> 13)
#define M4Pos ((GPIOD->IDR & H4Mask) >> 13)
#define M5Pos ((GPIOE->IDR & H5Mask) >> 6)

/*
 * TYPEDEFS
 */
typedef struct _MotorDataSet
{
	volatile uint32_t A;
	volatile uint32_t B;

	uint32_t CWLUTA[8];
	uint32_t CWLUTB[8];

	volatile uint16_t power;

	volatile int32_t coms;
	volatile uint8_t ccw;
} MotorDataSet;

typedef struct _MotorGlobal
{
	MotorDataSet M[5];

	uint32_t PhaseLUT[8];
	uint8_t EXTILUT[5];	// convert motor id to EXTI line

	int32_t maxPWM;
} MotorGlobal;

/*
 * DATA
 */
extern MotorGlobal motor;

/*
 * FUNCTION PROTOTYPES
 */
void 	MotorInit();
void	MotorEmergencyShutdown();
void	MotorEmergencyRecovery();
void	MotorSetPower(uint8_t id, uint16_t newPower);
void	MotorClearDriverFault(uint8_t id);

static __inline__ void MotorSetDir(uint8_t id, uint8_t ccw)
{
	motor.M[id].ccw = ccw;
}

#endif /* MOTOR_H_ */
