/*
 * usart1.c
 *
 *  Created on: 25.05.2011
 *      Author: AndreR
 */

#include "usart1.h"

#include "util/bit_banding.h"

#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_usart.h"
#include "misc.h"

USART1Data usart1;

#define ClearTXEI() BBPeriphReset(USART1_BASE + 0x0C, 7)
#define SetTXEI() BBPeriphSet(USART1_BASE + 0x0C, 7)
#define GetTXEI() BBPeriphGet(USART1_BASE + 0x0C, 7)
#define ClearRXI() BBPeriphReset(USART1_BASE + 0x0C, 5)
#define SetRXI() BBPeriphSet(USART1_BASE + 0x0C, 5)
#define GetRXI() BBPeriphGet(USART1_BASE + 0x0C, 5)

#define USART1_IDLE 0
#define USART1_RUNNING 1

void USART1_IRQHandler()
{
	uint32_t sr = USART1->SR;

	if(sr & USART_SR_TXE)	// enabled by TXEIE
	{
		// Transmitter empty, cleared by write to DR
		uint8_t data;

		if(FifoSyncedPopByte(&usart1.txFifo, &data) == 0)
		{
			USART1->DR = data;
		}
		else
		{
			ClearTXEI();
			usart1.state = USART1_IDLE;
		}
	}

	if(sr & USART_SR_RXNE)	// enabled by RXNEIE
	{
		// read data register not empty, cleared by: read DR
		uint8_t data = USART1->DR;

		FifoSyncedPushByte(&usart1.txFifo, data);
		USART1StartTx();

		if(data == '\r' || data == '\n')	//finish command
		{
			FifoPacketSyncedComplete(&usart1.rxFifo);
		}
		else
		{
			if(data == 127)	// backspace
				FifoPacketSyncedRevert(&usart1.rxFifo);
			else
				FifoPacketSyncedPushByte(&usart1.rxFifo, data);
		}
	}
}

uint8_t rxLockFifo(void* pData)
{
	pData = pData;

	uint8_t state = 0;

	if(GetRXI())
	{
		state = 1;
		ClearRXI();
	}

	return state;
}

void rxReleaseFifo(uint8_t state, void* pData)
{
	pData = pData;

	if(state)
		SetRXI();

	__asm__ volatile("" ::: "memory");
}

uint8_t txLockFifo(void* pData)
{
	pData = pData;

	uint8_t state = 0;

	if(GetTXEI())
	{
		state = 1;
		ClearTXEI();
	}

	return state;
}

void txReleaseFifo(uint8_t state, void* pData)
{
	pData = pData;

	if(state)
		SetTXEI();

	__asm__ volatile("" ::: "memory");
}

void USART1Init()
{
	FifoPacketSyncedInit(&usart1.rxFifo, usart1.rxData, USART1_RX_SIZE, &rxLockFifo, &rxReleaseFifo, 0);
	FifoSyncedInit(&usart1.txFifo, usart1.txData, USART1_TX_SIZE, &txLockFifo, &txReleaseFifo);

	usart1.state = USART1_IDLE;

	// init port
	GPIO_InitTypeDef gpioInit;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

	gpioInit.GPIO_Mode = GPIO_Mode_AF_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_9;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &gpioInit);

	gpioInit.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	gpioInit.GPIO_Pin = GPIO_Pin_10;
	GPIO_Init(GPIOA, &gpioInit);

	USART_InitTypeDef USARTInit;
	USARTInit.USART_BaudRate = 115200;
	USARTInit.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USARTInit.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USARTInit.USART_Parity = USART_Parity_No;
	USARTInit.USART_StopBits = USART_StopBits_1;
	USARTInit.USART_WordLength = USART_WordLength_8b;
	USART_Init(USART1, &USARTInit);

	NVIC_InitTypeDef NVICInit;
	NVICInit.NVIC_IRQChannel = USART1_IRQn;
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_USART1;	//3. lowest priority
	NVICInit.NVIC_IRQChannelSubPriority = 0x00;	//lowest priority
	NVIC_Init(&NVICInit);

	USART_Cmd(USART1, ENABLE);

	SetRXI();
}

void USART1StartTx()
{
	if(usart1.state == USART1_IDLE)
	{
		usart1.state = USART1_RUNNING;
		SetTXEI();
	}
}

