#ifndef __MOVEMENT_H__
#define __MOVEMENT_H__

#include "stm32f10x.h"
#include "util/pid.h"
#include "util/atmega.h"
#include "atmega/motor_prog/motor_data.h"

#define MOTOR_MODE_MANUAL		0
#define MOTOR_MODE_PID			1
#define MOTOR_MODE_AUTOMATIC	2

#define MOVEMENT_AXIS_X			0
#define MOVEMENT_AXIS_Y			1
#define MOVEMENT_AXIS_W			2
#define MOVEMENT_AXIS_V			3

typedef struct _MovementDataSet
{
	PID pid;
	TIM_TypeDef* pTIM;
	volatile int32_t lastTickCount;
	volatile int32_t lastEncCount;
	uint8_t logEnabled;
	uint8_t overCurrentCounter;
	uint32_t maxCurrent;
	uint32_t lastReaction;
} MovementDataSet;

typedef struct _MovementGlobal
{
	MovementDataSet M[5];

	uint8_t mode;

	struct
	{
		int32_t sumAx;
		int32_t sumAy;
		int32_t sumAw;
	} lis3;

	int32_t latest[4];
	int32_t target[4];

	struct
	{
		int32_t wPart;
		int32_t vPart;
	} wDiv;

	PID rotCompPid;
	int32_t rotCompOut;
	int32_t rotCompFeedback;

	ATMega atmega;
	MotorData atData;

	uint8_t logEnabled;
} MovementGlobal;

extern MovementGlobal movement;

void MovementInit();
void MovementTask(void* params);
void MovementSetManual(uint8_t id, int16_t power);
void MovementSetPidSetpoint(uint8_t id, int32_t sp);
void MovementSetAutomatic(int32_t vx, int32_t vy, int32_t vw, int32_t vv);
void MovementSetMotorPidParams(uint8_t id, int32_t Kp, int32_t Ki, int32_t Kd, int32_t slew);
void MovementSetMode(uint8_t mode);
void MovementSetLog(uint8_t id, uint8_t enable);
void MovementSetDribble(int32_t rpm);
void MovementSetFeedbackLog(uint8_t enable);

#endif
