/*
 * dect.h
 *
 *  Created on: 01.07.2010
 *      Author: AndreR
 */

#ifndef __NETWORK_H__
#define __NETWORK_H__

#include "util/fifo_packet_synced.h"
#include "network/inet.h"
#include "network/dect_interface.h"
#include "constants.h"

#include "FreeRTOS.h"
#include "semphr.h"

#define NETWORK_STATE_HEADLESS	0
#define NETWORK_STATE_ASSIGNED	1

#define PACKET_HEADER_SIZE 4
typedef union _PacketHeader
{
	struct
	{
		uint8_t cmd;
		uint8_t section;
		uint16_t len;   // data length only
	};

	uint8_t _data[PACKET_HEADER_SIZE];
} PacketHeader;

typedef struct _Network
{
	uint8_t state;

	uint8_t txProcessingBuf[NETWORK_TX_PROCESSING_SIZE];
	uint8_t rxProcessingBuf[NETWORK_RX_PROCESSING_SIZE];

	xSemaphoreHandle hNewPacket;
	xSemaphoreHandle hSendPacket;

	uint16_t ethernetIdent;

	struct	// own values used as source
	{
		MAC mac;
		IPv4Address ip;
		uint16_t port;
	} my;

	struct	// values used for multicast receive, send
	{
		MAC mac;			// 01-00-5E-00-00-2A
		IPv4Address ip;		// 225.42.42.42
		uint16_t port;		// 10041
		uint16_t serverPort;	// 10040
	} mcast;

	struct	// values used as unicast destination
	{
		MAC mac;
		IPv4Address ip;
		uint16_t port;
	} ucast;

	uint32_t cpuId[3];

	uint8_t botId;

	DECTStatistic lastStats;

	struct
	{
		uint32_t respTime;
		uint8_t respCounter;
		uint32_t lastSendTime;
		uint8_t robustness;
	} igmp;

	uint8_t enableTimeout;
} Network;

extern Network network;

void	NetworkInit();
void	NetworkTask(void* params);
int16_t	NetworkSendPacket(PacketHeader* pHeader, uint8_t* pData, uint8_t multicast);
int16_t NetworkSendIGMPv3MembershipReport(uint16_t numberOfGroupRecords, uint8_t* recordData, uint16_t recordDataLength);
int16_t NetworkSendICMP(uint8_t* pData, uint16_t dataLength, IPv4Address dstIP, MAC dstMAC);

#endif /* DECT_H_ */
