/*
 * spi1_2.c
 *
 *  Created on: 06.03.2011
 *      Author: AndreR
 */

#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "misc.h"

#include "spi1_2.h"
#include "constants.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

SPI spi1;
SPI spi2;

xSemaphoreHandle hNewPacket;

void DMA1_Channel2_IRQHandler()
{
	SPIDmaRxInterrupt(&spi1);
}

void DMA1_Channel4_IRQHandler()
{
	SPIDmaRxInterrupt(&spi2);
}

void SPI1Callback()
{
	signed portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

	xSemaphoreGiveFromISR(hNewPacket, &xHigherPriorityTaskWoken);

	if(xHigherPriorityTaskWoken == pdTRUE)
		taskYIELD();
}

void SPI2Callback()
{
	signed portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

	xSemaphoreGiveFromISR(hNewPacket, &xHigherPriorityTaskWoken);

	if(xHigherPriorityTaskWoken == pdTRUE)
		taskYIELD();
}

int16_t SPI1_2Init()
{
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);

	static uint8_t spi1TxData[SPI1_FIFO_SIZE];
	static uint8_t spi1RxData[SPI1_FIFO_SIZE];
	static uint8_t spi1DmaTxData[SPI1_DMA_SIZE];
	static uint8_t spi1DmaRxData[SPI1_DMA_SIZE];
	static uint8_t spi1CbData[SPI1_CB_SIZE];

	static uint8_t spi2TxData[SPI2_FIFO_SIZE];
	static uint8_t spi2RxData[SPI2_FIFO_SIZE];
	static uint8_t spi2DmaTxData[SPI2_DMA_SIZE];
	static uint8_t spi2DmaRxData[SPI2_DMA_SIZE];
	static uint8_t spi2CbData[SPI2_CB_SIZE];

	SPIData data1;
	data1.pDataTx = spi1TxData;
	data1.pDataRx = spi1RxData;
	data1.size = SPI1_FIFO_SIZE;
	data1.pDmaDataTx = spi1DmaTxData;
	data1.pDmaDataRx = spi1DmaRxData;
	data1.dmaSize = SPI1_DMA_SIZE;
	data1.pCbData = spi1CbData;
	data1.cbSize = SPI1_CB_SIZE;
	data1.dmaChRx = 2;
	data1.dmaChTx = 3;
	data1.pDma = DMA1;

	SPIData data2;
	data2.pDataTx = spi2TxData;
	data2.pDataRx = spi2RxData;
	data2.size = SPI2_FIFO_SIZE;
	data2.pDmaDataTx = spi2DmaTxData;
	data2.pDmaDataRx = spi2DmaRxData;
	data2.dmaSize = SPI2_DMA_SIZE;
	data2.pCbData = spi2CbData;
	data2.cbSize = SPI2_CB_SIZE;
	data2.dmaChRx = 4;
	data2.dmaChTx = 5;
	data2.pDma = DMA1;

	SPIInit(&spi1, SPI1, data1, &SPI1Callback);
	SPIInit(&spi2, SPI2, data2, &SPI2Callback);

	NVIC_InitTypeDef NVICInit;
	NVICInit.NVIC_IRQChannel = DMA1_Channel2_IRQn;
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_SPI1;
	NVICInit.NVIC_IRQChannelSubPriority = 0x00;
	NVIC_Init(&NVICInit);

	NVICInit.NVIC_IRQChannel = DMA1_Channel4_IRQn;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_SPI2;
	NVIC_Init(&NVICInit);

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;	// SCK, MISO, MOSI
	gpioInit.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &gpioInit);

	gpioInit.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init(GPIOB, &gpioInit);

	vSemaphoreCreateBinary(hNewPacket);

	return 0;
}

void SPI1_2Task(void* params)
{
	params = params;

	while(1)
	{
		xSemaphoreTake(hNewPacket, 10/portTICK_RATE_MS);

		SPINotifySlaveCallbacks(&spi1);
		SPINotifySlaveCallbacks(&spi2);
	}
}
