/*
 * usb.c
 *
 *  Created on: 23.05.2010
 *      Author: AndreR
 */

#include "stm32f10x.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_gpio.h"
#include "misc.h"

#include "usb.h"
#include "usb_lib.h"
#include "usb_vcom_desc.h"
#include "usb_vcom.h"

#include "util/bit_banding.h"

#include "constants.h"

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

USBData usb;

uint8_t vcomBufferRx[VIRTUAL_COM_PORT_DATA_SIZE];
uint8_t vcomBufferTx[VIRTUAL_COM_PORT_DATA_SIZE];

#define ClearCTRI() BBPeriphReset(APB1PERIPH_BASE + 0x5C00 + 0x40, 15)
#define SetCTRI()	BBPeriphSet(APB1PERIPH_BASE + 0x5C00 + 0x40, 15)
#define GetCTRI()	BBPeriphGet(APB1PERIPH_BASE + 0x5C00 + 0x40, 15)

uint8_t lockFifo(void* pData)
{
	pData = pData;

	uint8_t state = 0;

	if(GetCTRI())
	{
		state = 1;
		ClearCTRI();
	}

	return state;
}

void releaseFifo(uint8_t state, void* pData)
{
	pData = pData;

	if(state)
		SetCTRI();

	__asm__ volatile("" ::: "memory");
}

void USBStartTx()
{
	if(GetEPTxStatus(ENDP1) == EP_TX_VALID)
		return;	// transmission in progress

	uint16_t bytesRead = 0;

	FifoSyncedPop(&usb.txFifo, VIRTUAL_COM_PORT_DATA_SIZE, vcomBufferTx, &bytesRead);

	if(bytesRead != 0)
	{
		USB_SIL_Write(EP1_IN, vcomBufferTx, bytesRead);
		SetEPTxValid(ENDP1);
	}
}

void EP1_IN_Callback()	// called after successful transmission
{
	USBStartTx();
}

void EP3_OUT_Callback() // called after successful reception
{
	uint16_t dataLength = USB_SIL_Read(EP3_OUT, vcomBufferRx);

	FifoSyncedPush(&usb.txFifo, dataLength, vcomBufferRx);
	USBStartTx();

	for(uint16_t i = 0; i < dataLength; i++)
	{
		uint8_t data = vcomBufferRx[i];

		if(data == '\r' || data == '\n')	//finish command
		{
			FifoPacketSyncedComplete(&usb.rxFifo);
		}
		else
		{
			if(data == 127)	// backspace
				FifoPacketSyncedRevert(&usb.rxFifo);
			else
				FifoPacketSyncedPushByte(&usb.rxFifo, data);
		}
	}

	SetEPRxValid(EP3_OUT);
}

void USBInit()
{
	FifoPacketSyncedInit(&usb.rxFifo, usb.rxData, USB_RX_SIZE, &lockFifo, &releaseFifo, 0);
	FifoSyncedInit(&usb.txFifo, usb.txData, USB_TX_SIZE, &lockFifo, &releaseFifo);

	RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5);

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_Out_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_15;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOG, &gpioInit);

	USBEnable(0);

	NVIC_InitTypeDef NVICInit;
	NVICInit.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_USB;	//  priority
	NVICInit.NVIC_IRQChannelSubPriority = 0;	//lowest priority
	NVIC_Init(&NVICInit);

	USB_Init();

	SetCTRI();
}

void USBEnable(uint8_t enable)
{
	if(enable)
		GPIOG->BSRR = GPIO_Pin_15;
	else
		GPIOG->BRR = GPIO_Pin_15;
}
