/*
 * atmega.h
 *
 *  Created on: 09.03.2011
 *      Author: AndreR
 */

#ifndef __ATMEGA_H__
#define __ATMEGA_H__

#include "util/spi.h"

#include "FreeRTOS.h"
#include "semphr.h"

#define ATMEGA_BUF_SIZE 1050	// should be enough for all types (= 128 Words page size)

typedef struct _ATMega
{
	SPI* pSPI;
	uint8_t slaveId;

	xSemaphoreHandle hWriteSemaphore;
	xSemaphoreHandle hDataInUse;

	union
	{
		uint8_t buf[ATMEGA_BUF_SIZE];
		struct
		{
			uint8_t tx[ATMEGA_BUF_SIZE/2];
			uint8_t rx[ATMEGA_BUF_SIZE/2];
		};
	};
	uint8_t bufValid;

	uint8_t programming;

	GPIO_TypeDef* pResetPort;
	uint16_t resetPin;

	uint8_t signature[3];
	uint8_t fuses[3]; // 0 = low, 1 = high, 2 = extended
} ATMega;

int16_t ATMegaInit(ATMega* pAT, SPI* pSPI, GPIO_TypeDef* pCSPort, uint16_t csPin, GPIO_TypeDef* pResetPort, uint16_t resetPin);
int16_t ATMegaExchange(ATMega* pAT, uint8_t* pMosiData, uint8_t* pMisoData, uint16_t dataLength);
int16_t ATMegaEnterProgramming(ATMega* pAT);
int16_t ATMegaExitProgramming(ATMega* pAT);
int16_t ATMegaReadSignature(ATMega* pAT);
int16_t ATMegaReadFuses(ATMega* pAT);
int16_t ATMegaWriteFuses(ATMega* pAT, uint8_t low, uint8_t high, uint8_t extended);
int16_t ATMegaChipErase(ATMega* pAT);
int16_t ATMegaLoadProgram(ATMega* pAT, const uint8_t* pData, uint16_t dataSize);
int16_t ATMegaVerifyProgram(ATMega* pAT, const uint8_t* pData, uint16_t dataSize);
int16_t ATMegaUpdateProgram(ATMega* pAT, const uint8_t* pData, uint16_t dataSize, uint8_t* pUpdated);

#endif
