/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */

package edu.dhbw.mannheim.tigers.robotcontrolutility.model;

import java.util.HashMap;
import net.java.games.input.Component;
import net.java.games.input.Controller;


/**
 * This class polls the Controller and passes components and to GamePadInterpreter.
 * 
 * @author Lukas
 */
public class ControllerReader
{
	
	// --------------------------------------------------------------------------
	// --- instance variables ---------------------------------------------------
	// --------------------------------------------------------------------------
	private HashMap<String, String>	currentConfig;
	private float							THRESHOLD_BTN	= 0.3f;
	private float							THRESHOLD_ANALOG = 0.03f;
	private boolean						poll		= false;
	private ControllerInterpreter		interpreter;
	private Controller					controller;
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	public ControllerReader(Controller controller)
	{
		this.controller = controller;
		this.interpreter = new ControllerInterpreter();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public boolean startPolling(String serverName, int portNumber)
	{
		PollingThread myPollingThread = new PollingThread();
		
		if (interpreter.start(serverName, portNumber))
		{
			interpreter.setCurrentConfig(currentConfig);
			myPollingThread.start();
			poll = true;
		}
		return poll;
	}
	

	public void stopPolling()
	{
		poll = false;
		interpreter.stop();
	}
	

	public void setCurrentConfig(HashMap<String, String> currentConfig)
	{
		this.currentConfig = currentConfig;
	}
	
	private class PollingThread extends Thread
	{
		public void run()
		{
			while (poll)
			{
				try
				{
					Thread.sleep(50);
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
				controller.poll();
				int i = 0;
				for (Component comp : controller.getComponents())
				{
					i++;
					// System.out.println(i);
					// --- get POV components ---
					if (comp.getIdentifier() == Component.Identifier.Axis.POV)
					{
						if (comp.getPollData() != 0)
						{
							// System.out.println("a");
							float povDir = comp.getPollData();
							POVToButton pov = new POVToButton(comp, povDir);
							// System.out.println("povPolling: "+pov.getName()+" value: "+pov.getPollData());
							interpreter.interpret(pov);
						}
					} else
					{
						// --- Button or Axis ---
						if (comp.isAnalog())
						{
							// --- Axis ---
							float pollData = comp.getPollData();
							if (pollData >= 0)
							{
								// --- positive Axis ---
								if (pollData > THRESHOLD_ANALOG)
								{
									// System.out.println("b");
									PositiveAxis pAxis = new PositiveAxis(comp);
									interpreter.interpret(pAxis);
								}
							} else
							{
								// --- negative Axis ---
								if (pollData < -THRESHOLD_ANALOG)
								{
									// System.out.println("d");
									NegativeAxis nAxis = new NegativeAxis(comp);
									interpreter.interpret(nAxis);
								}
							}
							
						} else
						{
							// --- Button ---
							if (comp.getPollData() > THRESHOLD_BTN)
							{
								// System.out.println("c");
								interpreter.interpret(comp);
							}
						}
					}
				}
				interpreter.fillActionMapWithNulls();
			}
		}
	}
}
