/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */

package edu.dhbw.mannheim.tigers.robotcontrolutility.model;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.InetSocketAddress;
import java.net.NoRouteToHostException;
import java.net.Socket;
import java.net.SocketTimeoutException;


/**
 * This class sends commands to RCU-Server in Sumatra.
 * 
 * @author Lukas
 * 
 */

public class SendingThread extends Thread
{
	
	// --------------------------------------------------------------------------
	// --- instance variables ---------------------------------------------------
	// --------------------------------------------------------------------------
	private final int				SLEEP			= 20;
	private ObjectOutputStream	printWriter	= null;
	private Socket					socket		= null;
	private ActionCommand		cmd			= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	public SendingThread(String serverName, int port) throws IOException, SocketTimeoutException, NoRouteToHostException
	{
		
		socket = new Socket();
		socket.connect(new InetSocketAddress(serverName, port), 500);
		printWriter = new ObjectOutputStream(socket.getOutputStream());
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public void run()
	{
		while (!this.isInterrupted())
		{
			if (cmd != null)
			{
				try
				{
					send(cmd);
				} catch (IOException e)
				{
					System.out.println("No more Connection to Server");
					break;
				}
			}
			try
			{
				Thread.sleep(SLEEP);
			} catch (InterruptedException e)
			{
				break;
			}
		}
		this.close();
	}
	

	private void send(ActionCommand cmd) throws IOException
	{
		printWriter.writeObject(cmd);
		printWriter.flush();
	}
	

	public void close()
	{
		if (!this.isInterrupted())
		{
			this.interrupt();
		}
		if (socket.isConnected())
		{
			try
			{
				socket.close();
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}
	}
	

	public void setNewCmd(ActionCommand newCmd)
	{
		this.cmd = newCmd;
	}
}
