/*
 * *********************************************************
 * Copyright (c) 2009 DHBW Mannheim - Tigers Mannheim
 * Project: tigers-robotControlUtility
 * Date: 19.11.2010
 * Authors: Clemens Teichmann <clteich@gmx.de>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.robotcontrolutility.view.actions;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JTextField;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.robotcontrolutility.model.NegativeAxis;
import edu.dhbw.mannheim.tigers.robotcontrolutility.model.POVToButton;
import edu.dhbw.mannheim.tigers.robotcontrolutility.model.PositiveAxis;
import edu.dhbw.mannheim.tigers.robotcontrolutility.presenter.AControllerPresenter;
import edu.dhbw.mannheim.tigers.robotcontrolutility.presenter.RCUPresenter;
import edu.dhbw.mannheim.tigers.robotcontrolutility.view.MainFrame;

import net.java.games.input.Component;
import net.java.games.input.Controller;


/**
 * @author Clemens
 * 
 */
public class ButtonSelectAction implements MouseListener, Runnable
{
	// --------------------------------------------------------------------------
	// --- instance variables ---------------------------------------------------
	// --------------------------------------------------------------------------
	
	private Controller				controller;
	private Component[]				comps;
	private JTextField				myTextField;
	private AControllerPresenter	controllerPresenter;
	private Thread						buttonSelectThread;
	
	private static final int		DELAY		= 50;															// 50ms delay
	private static final double	THRESHOLD	= 0.3;															// Threshold
																															

	// --- time for timeout ---
	private long						startSystemTime;
	
	// --- logger ---
	private Logger						log		= Logger.getLogger(RCUPresenter.class.getName());
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	
	public ButtonSelectAction(JTextField selectedTextField, Controller newController,
			AControllerPresenter newControllerPresenter)
	{
		super();
		myTextField = selectedTextField;
		controller = newController;
		controllerPresenter = newControllerPresenter;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	
	public void mouseClicked(MouseEvent e)
	{
		log.info("Please press selected Button");
		buttonSelectThread = new Thread(this);
		buttonSelectThread.setName("ButtonSelectThread");
		// --- set start time ---
		startSystemTime = System.currentTimeMillis();
		// --- start Thread ---
		buttonSelectThread.start();
	}
	

	public void mouseEntered(MouseEvent e)
	{
	}
	

	public void mouseExited(MouseEvent e)
	{
	}
	

	public void mousePressed(MouseEvent e)
	{
	}
	

	public void mouseReleased(MouseEvent e)
	{
	}
	

	public void run()
	{
		// --- check variable ---
		boolean componentPressed = false;
		// --- get all components of current controller ---
		comps = controller.getComponents();
		
		// --- run until thread interrupted by user or timeout ---
		while (!Thread.interrupted())
		{
			// --- add timer to MainFrame title
			MainFrame.getInstance().setMainFrameTitle(
					Math.round((6 - (System.currentTimeMillis() - startSystemTime) / 1000)) + "sec left to press button");
			try
			{
				// --- wait a while ----
				Thread.sleep(DELAY);
			} catch (InterruptedException e)
			{
				// --- TimeOut ---
				log.warn("ButtenSelect Timeout");
				System.out.println(e.getMessage());
			}
			
			// --- update the controllers components ---
			controller.poll();
			
			// --- search trough all components ---
			for (int idx = 0; idx < comps.length; idx++)
			{
				
				// --- get POV components ---
				if (comps[idx].getIdentifier() == Component.Identifier.Axis.POV)
				{
					// --- if POV component pressed ---
					if (comps[idx].getPollData() != 0)
					{
						float povDir = comps[idx].getPollData();
						
						// --- handle POV as button ---
						POVToButton pov = new POVToButton(comps[idx], povDir);
						// --- add pov to config ---
						controllerPresenter.onNewSelectedButton(pov, myTextField.getName());
						// --- set textfield text ---
						myTextField.setText(pov.getIdentifier().toString());
						log.info("Component " + comps[idx].getIdentifier().toString() + " selected");
						componentPressed = true;
					}
				}
				// --- get other components ---
				else
				{
					// --- get Buttons and positive Axis ---
					if (!(comps[idx].getPollData() <= THRESHOLD))
					{
						// --- positive Axis ---
						if (comps[idx].isAnalog())
						{
							PositiveAxis pAxis = new PositiveAxis(comps[idx]);
							controllerPresenter.onNewSelectedButton(pAxis, myTextField.getName());
							myTextField.setText(pAxis.getIdentifier().toString());
							log.info("Component " + pAxis.getIdentifier().toString() + " selected");
							componentPressed = true;
						}
						// --- Button ---
						else
						{
							controllerPresenter.onNewSelectedButton(comps[idx], myTextField.getName());
							myTextField.setText(comps[idx].getIdentifier().toString());
							log.info("Component " + comps[idx].getIdentifier().toString() + " selected");
							componentPressed = true;
						}
					}
					
					// --- get negative Axis ---
					if (!(comps[idx].getPollData() >= -THRESHOLD))
					{
						NegativeAxis nAxis = new NegativeAxis(comps[idx]);
						controllerPresenter.onNewSelectedButton(nAxis, myTextField.getName());
						myTextField.setText(nAxis.getIdentifier().toString());
						log.info("Component " + nAxis.getIdentifier().toString() + " selected");
						
						componentPressed = true;
					}
				}
			}
			// --- end thread if user pressed component or timeout ---
			if (componentPressed || (System.currentTimeMillis() - startSystemTime) > 6000)
			{
				// --- reset MainFrame title ---
				MainFrame.getInstance().setMainFrameTitle("");
				Thread.currentThread().interrupt();
			}
		}
	}
}