/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */
package testServer;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import edu.dhbw.mannheim.tigers.robotcontrolutility.model.ActionCommand;


/**
 * This class handles the RCC for one bot (just Test)
 * 
 * @author Lukas
 * 
 */
public class RCReceiver implements Runnable
{
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	
	
	// Connection
	private final Thread						thread;
	private Socket								socket				= null;
	private ServerSocket						serverSocket		= null;
	private int									port;
	
	// Input
	private ObjectInputStream				inputReader			= null;
	private ActionCommand					clientCommand		= null;
	private ActionCommand					oldCommand			= null;
	
	// Interpretation
	private Bot									bot					= null;
	private final Interpreter				interpreter;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public RCReceiver(int port, Bot bot)
	{
		this.port = port;
		this.bot = bot;
		this.thread = new Thread(this, "RCReceiver_" + bot + "|" + port);
		interpreter = new Interpreter();
	}
	

	// --------------------------------------------------------------------------
	// --- method(s) ------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void run()
	{
		while (!thread.isInterrupted())
		{
			try
			{
				// --- start a new server and wait for a client ---
				if (serverSocket != null)
				{
					serverSocket.close();
					socket.close();
				}
				
				serverSocket = new ServerSocket(port);
				
				socket = serverSocket.accept();
				

				// --- init input-reader ---
				inputReader = new ObjectInputStream(socket.getInputStream());
				

				// --- while client is connected to server -> do your work ---
				while ((clientCommand = (ActionCommand) inputReader.readObject()) != null)
				{
					if(!clientCommand.equals(oldCommand))
					{
						interpreter.interpret(clientCommand);
						oldCommand = clientCommand;
					}
				}
			} catch (SocketException e)
			{
				System.out.println("Client disconnected!"+
				"\nWaiting for a new Connection...");
			} catch (IOException e)
			{
				System.out.println("Client disconnected!"+
				"\nWaiting for a new Connection...");
			} catch (ClassNotFoundException e)
			{
				System.err.println("Received Object is no ActionCommand!");
				e.printStackTrace();
			}
			
		}
	}
	
	
	public void startReceiver()
	{
		thread.start();
	}
	

	public void close()
	{
		thread.interrupt();
		try
		{
			if (serverSocket != null)
			{
				serverSocket.close();
			}
			if (socket != null)
			{
				socket.close();
				System.out.println("Socket closed!");
			}
		} catch (IOException e)
		{
			System.err.println("Expected IOException was thrown.");
		}
	}
	
	
	@Override
	public String toString()
	{
		return "RCReceiver " + bot;
	}
}
