package edu.dhbw.mannheim.tigers.sim.controller;

import edu.dhbw.mannheim.tigers.sim.controller.MainController.Type;
import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.ISimEngine;
import edu.dhbw.mannheim.tigers.sim.engine.SimEngine;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ESimEngineMode;
import edu.dhbw.mannheim.tigers.sim.engine.observer.ISimEngineObservable;


/**
 * This {@link ASubController} provides access to the {@link SimEngine}
 * 
 * @author Gero
 */
class SimEngineController extends ASubController implements ISimEngineController {

	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private ISimEngine engine;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	SimEngineController(MainController main) {
		super(main, Type.ENGINE);
	}
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public boolean createSimEngine(ESimEngineMode mode) {
		boolean deletedOld = false;
		
		if (engine != null) {
			deletedOld = true;
			engine.stop();
			engine.shutdown();
		}
		
		engine = new SimEngine(mode, null);
			
		engine.addObservers((InputController) main.getController(Type.INPUT));
		
		return deletedOld;
	}
	
	
	@Override
	public void prepareAppExit() {
		
		if (engine != null) {
			engine.stop();
			
			engine.waitForCleanup();
			
			engine = null;
		}
	}


	@Override
	public boolean verifyAppExit() {
		boolean mayExit = true;
		
		if (engine != null && !engine.getState().equals(ESimEngineState.STOPPED)) {
			mayExit = main.getInputController().getGUI().verifyEndOfSimulation();
		}
		
		return mayExit;
	}
	
	
	ISimEngine getEngine() {
		return engine;
	}


	@Override
	public ISimEngine getSimEngine() {
		return getEngine();
	}


	@Override
	protected void onUpdate(ISimEngineObservable observable, ESimEngineState event) {/*DO NOTHING*/}
}
