package edu.dhbw.mannheim.tigers.sim.engine;


import edu.dhbw.mannheim.tigers.sim.engine.modes.ASimEngineMode;
import edu.dhbw.mannheim.tigers.sim.engine.observer.ISimEngineObservable;


/**
 * This interface represents the simulators core functionality provided by the {@link SimEngine} to
 * the rest of the application. It can be referred to as the opposite of the {@link ICore}, which
 * represents the {@link SimEngine} to the <i>inner parts</i> of the application.
 * 
 * @author Gero
 * 
 */
public interface ISimEngine extends IBasicSimEngine, ISimEngineObservable {
	
	/**
	 * Defines, how many cycles the simulation should try to reach per second.
	 * Actually, it will never really reach this value but be marginal below it,
	 * even if the machine is good enough.
	 * 
	 * @param newCycleRate
	 */
	public void setCyclesPerSecond(int newCycleRate);
	

	/**
	 * @return @see {@link ACycleControlledEngine#getCurrentCycleRate()}
	 */
	public float getCurrentCycleRate();
	

	/**
	 * The desired frames per second. This value could never be higher then {@link #getCurrentCycleRate()} and
	 * should be a divider of it to reach highest possible accuracy to the desired value.
	 * 
	 * @param newFrameRate
	 */
	public void setFramesPerSecond(int newFrameRate);
	

	/**
	 * The value set by @see #setFramesPerSecond(int)
	 * 
	 * @return
	 */
	public int getFramesPerSecond();
	

	/**
	 * @return The {@link ASimEngineMode} the SimEngine is running at
	 */
	public ASimEngineMode getMode();
	

	/**
	 * @return The {@link ESimEngineState} the SimEngine is in
	 */
	public ESimEngineState getState();
	

	/**
	 * Closes and cleanup the simulation before application shutdown
	 */
	public void shutdown();
	

	/**
	 * This methods waits until the {@link SimEngine} is ready with cleanup, or immediately returns if this has already
	 * been done
	 */
	public void waitForCleanup();
}
