package edu.dhbw.mannheim.tigers.sim.engine.com.bot;


import edu.dhbw.mannheim.tigers.sim.engine.com.IRoboCupMatchControlProvider;
import edu.dhbw.mannheim.tigers.sim.engine.com.IStartableInterface;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBotControl;


/**
 * Abstract base class for BotControllers for all kinds of bots.
 * 
 * @author Gero
 * 
 * @param <B> {@link IBotControl} implementation
 * @param <C> command base type
 * @param <S> status base type
 */
public abstract class ABotController<B extends IBotControl, C, S> implements IStartableInterface, IBotController<C> {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Bot
	protected final B										bot;
	
	// GameControlProvider
	protected final IRoboCupMatchControlProvider	matchControlProvider;
	
	// Comm
	private ABotCommunicator<?, ?>					comm;
	private boolean										blocked	= false;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------	
	/**
	  * @see ABotController
	  * @param bot
	  * @param matchControlProvider
	  */
	public ABotController(B bot, IRoboCupMatchControlProvider matchControlProvider) {
		
		this.bot = bot;
		this.matchControlProvider = matchControlProvider;
	}
	

	public void setBotCommunicator(ABotCommunicator<?, ?> newCommunicator) {
		this.comm = newCommunicator;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Set first via {@link #setBotCommunicator(ABotCommunicator)}!
	 * 
	 * @return The {@link ABotController}s {@link ABotCommunicator}
	 */
	protected ABotCommunicator<?, ?> getBotCommunicator() {
		return this.comm;
	}
	

	@Override
	public boolean isBlocked() {
		return blocked;
	}
	

	@Override
	public void setBlocked(boolean blocked) {
		this.blocked = blocked;
	}
}
