package edu.dhbw.mannheim.tigers.sim.engine.com.bot;

import edu.dhbw.mannheim.tigers.sim.engine.model.bot.BotID;


/**
 * This interface belongs to the botcom-trio and offers some of the {@link ABotCommunicator} capabilities
 * to {@link ABotCommandReceiver} and {@link ABotStatusTransmitter}.
 * 
 * @author Gero
 * 
 * @param <C> Command-base class
 */
public interface ICommunicator<C> {
	
	/**
	 * Starts the communicator
	 */
	public void start();
	

	/**
	 * Stops the communicator
	 */
	public void stop();
	

	/**
	 * A matter of indirection: Called by child-classes to pass a new received command to the controller
	 * @param cmd
	 */
	public void newCommand(C cmd);
	

	/**
	 * Called to notify that the state of a child-comm has changed
	 */
	public void notifyShutdown();
	
	
	/**
	 * @return The bots unique ID
	 */
	public BotID getBotId();
}
