package edu.dhbw.mannheim.tigers.sim.engine.com.bot.ct;


import java.io.IOException;
import java.io.InputStream;

import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ABotCommandReceiver;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ICommunicator;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ACommand;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.CommandConstants;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.CommandFactory;


/**
 * The CT-bot implementation of the {@link ABotCommandReceiver}
 * 
 * @see CTBotCommunicator
 * @see CTBotCommandReceiver
 * 
 * @author Gero, Malte
 */
public class CTBotCommandReceiver extends ABotCommandReceiver<ACommand> {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Input
	private InputStream	in;
	
	// Message de-serialization
	private byte			header[]			= new byte[CommandConstants.HEADER_SIZE];
	private int				headerPos		= 0;
	
	private byte			body[]			= null;
	private int				bodyPos			= 0;
	
	private boolean		waitForHeader	= true;
	private boolean		waitForData		= false;
	private boolean		nextIsCoded		= false;
	private ACommand		cmd				= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see CTBotCommandReceiver
	 * @param communicator
	 * @param inputStream
	 * @param name
	 */
	public CTBotCommandReceiver(ICommunicator<ACommand> communicator, InputStream inputStream, String name) {
		super(communicator, name);
		
		this.in = inputStream;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void interpret() throws IOException {
		
		int c = in.read();
		if (c == -1)
			throw new IOException();
		
		if (c == 0x7E) // start flag
		{
			waitForHeader = true;
			waitForData = false;
			header = new byte[CommandConstants.HEADER_SIZE];
			headerPos = 0;
			
			return;
		}
		
		if (c == 0x7D) // escape flag
		{
			nextIsCoded = true;
			return;
		}
		
		if (nextIsCoded) {
			nextIsCoded = false;
			
			if (c == 0x5E)
				c = 0x7E;
			
			if (c == 0x5D)
				c = 0x7D;
		}
		
		if (waitForHeader) {
			header[headerPos++] = (byte) c;
			
			if (headerPos < CommandConstants.HEADER_SIZE)
				return;
			
			// header is complete
			cmd = CommandFactory.createEmptyPacket(header);
			headerPos = 0;
			header = new byte[CommandConstants.HEADER_SIZE];
			
			if (cmd == null) // unknown command
			{
				return;
			}
			
			if (cmd.getDataLength() == 0) // packet already complete
			{
				comm.newCommand(cmd);
			} else {
				waitForHeader = false;
				waitForData = true;
				bodyPos = 0;
				body = new byte[cmd.getDataLength()];
			}
			
			return;
		}
		
		if (waitForData) {
			body[bodyPos++] = (byte) c;
			
			if (bodyPos < cmd.getDataLength())
				return;
			
			// data complete
			cmd.setData(body);
			comm.newCommand(cmd);
			bodyPos = 0;
			waitForData = false;
			waitForHeader = true;
		}
	}
	

	@Override
	protected void closeInput() throws IOException {
		if (in != null) {
			in.close();
			in = null;
		}
	}
}