package edu.dhbw.mannheim.tigers.sim.engine.com.bot.ct;


import java.util.List;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.engine.com.IRoboCupMatchControlProvider;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ABotController;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.BotID;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ct.CTBotStatus;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ct.ICTBotControl;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ACommand;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.CommandConstants;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ct.CTPIDHistory;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ct.CTSetPID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ct.CTSetSpeed;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ct.CTStatus;


/**
 * Interprets the incoming commands and controls the given simulated-bot.
 * 
 * @author Gero, Gunther, Malte
 */
public class CTBotController extends ABotController<ICTBotControl, ACommand, CTStatus> implements ICTBotController {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private final Logger			log			= Logger.getLogger(getClass());
	
	// Com
	private CTBotCommunicator	communicator;
	

	// CTBot status values
	private static final float	MAX_SPEED	= 0.7f;
	private int						kdL			= 0;										// differential
	private int						kdR			= 0;
	private int						kiL			= 0;										// integral
	private int						kiR			= 0;
	private int						kpL			= 0;										// ?
	private int						kpR			= 0;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see CTBotController
	 * @param name
	 * @param port
	 * @param matchControlProvider
	 * @param bot
	 */
	public CTBotController(String name, int port, IRoboCupMatchControlProvider matchControlProvider, ICTBotControl bot) {
		super(bot, matchControlProvider);
		
		setBotCommunicator(communicator = new CTBotCommunicator(port, this));
		communicator.initialize();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void start() {
		communicator.start();
	}
	

	@Override
	public void stop() {
		communicator.stop();
	}
	

	@Override
	public void onConnect() {
		// This page intentionally left blank
	}
	

	@Override
	public void onDisconnect() {
		bot.setSpeed(0, 0);
	}
	

	@Override
	public void processNewCommand(ACommand cmd) {
		
		if (isBlocked()) {
			return; // Should command be executed?
		}
		
		switch (cmd.getCommand()) {
			
			case CommandConstants.CMD_CT_CALIBRATE:
				log.debug("Got Calibrate-command.");
				break;
			

			case CommandConstants.CMD_CT_SET_PID:
				CTSetPID pid = (CTSetPID) cmd;
				
				// Convert to mm/s
				kdL = (int) (pid.getKd()[0] * 1000);
				kdR = (int) (pid.getKd()[1] * 1000);
				
				kiL = (int) (pid.getKi()[0] * 1000);
				kiR = (int) (pid.getKi()[1] * 1000);
				
				kpL = (int) (pid.getKp()[0] * 1000);
				kpR = (int) (pid.getKp()[1] * 1000);
				
				if (communicator != null) {
					communicator.setStatusInterval(pid.getDelay());
				}
				log.debug("Got PID-command.");
				break;
			

			case CommandConstants.CMD_CT_SET_SPEED:
				CTSetSpeed cmdspeed = (CTSetSpeed) cmd;
				int l = Math.round(cmdspeed.getLeft() * 1000); // Convert to [mm/s]
				int r = Math.round(cmdspeed.getRight() * 1000);
				bot.setSpeed(l, r);
				break;
			
			case CommandConstants.CMD_CT_STATUS:
			default:
				break;
			
		}
		
	}
	

	@Override
	public List<ACommand> getBotStatus(List<ACommand> store) {
		store.add(new CTStatus(0, 0, kpL, kpR, kiL, kiR, kdL, kdR, MAX_SPEED, MAX_SPEED, MAX_SPEED, MAX_SPEED, 0));
		return store;
	}
	

	@Override
	public BotID getID() {
		return bot.getID();
	}
	

	@Override
	public CTPIDHistory[] getPIDHistory() {
		final CTBotStatus st = bot.getBotStatus();
		
		final int curSpeedL = Math.round(st.currentSpeedL); // mm/s
		final int curSpeedR = Math.round(st.currentSpeedR); // mm/s
		
		final int tarSpeedL = Math.round(st.targetSpeedL); // mm/s
		final int tarSpeedR = Math.round(st.targetSpeedR); // mm/s
		

		final CTPIDHistory[] pidHs = new CTPIDHistory[2];
		pidHs[0] = new CTPIDHistory(0, tarSpeedL, curSpeedL, kpL, kiL, kdL);
		pidHs[1] = new CTPIDHistory(1, tarSpeedR, curSpeedR, kpR, kiR, kdR);
		return pidHs;
	}
}
