package edu.dhbw.mannheim.tigers.sim.engine.com.bot.tiger;


import java.io.IOException;
import java.net.DatagramPacket;

import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ABotCommandReceiver;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ABotCommunicator;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ICommunicator;
import edu.dhbw.mannheim.tigers.sim.util.network.UnicastUDPTransceiver;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ACommand;


/**
 * The {@link ABotCommandReceiver}-implementation for the Tiger-bot. Reads commands and passes them
 * up to the {@link ABotCommunicator}
 * 
 * @see TigerBotCommunicator2
 * @see TigerBotStatusTransmitter
 * 
 * @author Gero
 */
public class TigerBotCommandReceiver extends ABotCommandReceiver<ACommand> {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Comm
	private static final int				BUFFER_SIZE	= 1500;
	
	private final UnicastUDPTransceiver	transceiver;
	
	// Message de-serialization
	private final TigerCommandParser		parser;
	
	private volatile long					lastPacketReceived;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see TigerBotCommandReceiver
	 * @param communicator
	 * @param inputStream
	 * @param name
	 */
	public TigerBotCommandReceiver(ICommunicator<ACommand> communicator, UnicastUDPTransceiver transceiver, String name) {
		super(communicator, name);
		
		this.transceiver = transceiver;
		this.parser = new TigerCommandParser();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void interpret() throws IOException {
		byte[] buffer = new byte[BUFFER_SIZE];
		DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
		packet = transceiver.receive(packet);
		
		if (packet != null) {
			lastPacketReceived = System.nanoTime();
			
			ACommand cmd = parser.readCommand(buffer);
			if (cmd != null) {
				comm.newCommand(cmd);
			}
		}
	}
	
	
	long getLastPacketReceived()
	{
		return lastPacketReceived;
	}
	

	@Override
	protected void closeInput() throws IOException {
		transceiver.cleanup();
		
		lastPacketReceived = 0;
	}
}
