package edu.dhbw.mannheim.tigers.sim.engine.delegates;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;

import com.jme.input.InputHandler;
import com.jme.input.KeyInput;
import com.jme.input.action.InputAction;
import com.jme.input.action.InputActionEvent;
import com.jme.renderer.ColorRGBA;
import com.jme.renderer.Renderer;
import com.jme.scene.BillboardNode;
import com.jme.scene.Spatial.LightCombineMode;
import com.jmex.angelfont.BitmapFont;
import com.jmex.angelfont.BitmapFont.Align;
import com.jmex.angelfont.BitmapFontLoader;
import com.jmex.angelfont.BitmapText;
import com.jmex.angelfont.Rectangle;

import edu.dhbw.mannheim.tigers.sim.engine.ICore;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.BotID;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBasicBotModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.modes.IMatchControlProvider;


/**
 * This class should provide the bots with some kind of marking, but it is not working though... :-/
 * 
 * @author Gero
 */
public class BotLabelDelegate extends ASimDelegate {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	
	private final IMatchControlProvider					botControlProvider;
	private final HashMap<BotID, BotLabel>				billboardNodes;
	
	private static final String							FONT_PATH = "resources/textures/couriernew.fnt";
	private static final String							TEXTURE_PATH = "resources/textures/couriernew_0.png";
	
	private boolean											showTextLabel	= false;
	private BitmapFont										font				= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see BotLabelDelegate
	 * @param core
	 * @param botControlProvider
	 */
	public BotLabelDelegate(ICore core, IMatchControlProvider botControlProvider) {
		super(core);
		this.botControlProvider = botControlProvider;
		this.billboardNodes = new HashMap<BotID, BotLabel>();
		

		// TODO Marian, Paul: Add logs/safety-checks!!! (Gero)
		File fontFile			= new File(FONT_PATH);
		File textureFile		= new File(TEXTURE_PATH);
		
		try {
			font = BitmapFontLoader.load(fontFile.toURI().toURL(), textureFile.toURI().toURL());
		} catch (MalformedURLException err) {
			// TODO Auto-generated catch block
			err.printStackTrace();
		} catch (IOException err) {
			// TODO Auto-generated catch block
			err.printStackTrace();
		}
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void initSimulation() {
		InputHandler input = core.getInputHandler();
		showBillBoards();
		
		input.addAction(new InputAction() {
			@Override
			public void performAction(InputActionEvent evt) {
				
				if (!evt.getTriggerPressed()) { // Trigger released
					if (!showTextLabel) {
						showBillBoards();
						showTextLabel = true;
					} else {
						hideBillBoards();
						showTextLabel = false;
					}
				}
			}
		}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_F2, InputHandler.AXIS_NONE, false);
	}
	

	@Override
	public void update(float interpolation, float tpc, float cps) {
		if (core.getCurrentCycle() % 50 == 0) {
			
			if (showTextLabel) {
				for (IBasicBotModelControl control : botControlProvider.getBotControls()) {
					BotID id = control.getID();
					BotLabel label = billboardNodes.get(id);
					if (label == null) {
						label = generateBotLabel(control);
						billboardNodes.put(id, label);
					}
					
					label.updateText(control.getInfoText());
				}
			}
		}
	}
	

	private void showBillBoards() {
		for (IBasicBotModelControl control : botControlProvider.getBotControls()) {
			BotLabel label = generateBotLabel(control);
			billboardNodes.put(control.getID(), label);
		}
	}
	

	private void hideBillBoards() {
		for (BotLabel control : billboardNodes.values()) {
			control.bNode.removeFromParent();
		}
		billboardNodes.clear();
	}
	

	private BotLabel generateBotLabel(IBasicBotModelControl control) {
		
		BitmapText bt = new BitmapText(font, false);
		bt.setName("" + control.getID().getIntValue());
		// set box directly over bot and make it 6 times the radius of the bot big
		bt.setBox(new Rectangle((control.getRadius() * -3), (control.getHeight() * 1.5f), (control.getRadius() * 6), 0));
		bt.setAlignment(Align.Center);
		bt.setDefaultColor(ColorRGBA.cyan);
		bt.setSize(100);
		bt.setText("" + control.getID().getIntValue());
		
		// important! text wont show if not set
		bt.setRenderQueueMode(Renderer.QUEUE_TRANSPARENT);
		bt.update();
		
		BillboardNode bNode = new BillboardNode("BillBoard " + control.getID().getIntValue()); // Zeigt richtung
																															// Bildschirm
		bNode.attachChild(bt);
		control.getNode().attachChild(bNode);
		
		return new BotLabel(bNode, bt);
	}
	
	private static class BotLabel {
		private final BillboardNode	bNode;
		private final BitmapText		bText;
		private String lastText = "";
		
		
		BotLabel(BillboardNode bNode, BitmapText bText) {
			this.bNode = bNode;
			this.bText = bText;
		}
		
		
		/**
		 * @param newText
		 * @return If anything was changed
		 */
		private boolean updateText(String newText) {
			if (!lastText.equals(newText)) {
				bText.setText(bText.getName() + "\n" + newText);
				bText.setSize(80);
				bText.setLightCombineMode(LightCombineMode.Off);
				bText.update();
				
				lastText = newText;
				return true;
			}
			
			return false;
		}
	}
	
}
