package edu.dhbw.mannheim.tigers.sim.engine.delegates;


import com.jme.input.InputHandler;
import com.jme.input.KeyInput;
import com.jme.input.action.InputAction;
import com.jme.input.action.InputActionEvent;
import com.jme.renderer.Renderer;
import com.jme.scene.Node;
import com.jme.scene.Spatial.CullHint;
import com.jme.scene.Text;
import com.jmex.physics.DynamicPhysicsNode;

import edu.dhbw.mannheim.tigers.sim.engine.ICore;
import edu.dhbw.mannheim.tigers.sim.engine.model.ModelHelper;
import edu.dhbw.mannheim.tigers.sim.engine.model.SimPhysicsPicker;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ABotModel;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.EBotModelType;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ct.ICTBotControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.parts.KickMode;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.tiger.ITigerBotControl;


/**
 * This delegate lets the user "enter" a bot and steer him to drive around (mainly for testing purposes)
 * 
 * @author Gero, Malte
 */
public class ControlBotDelegate extends ASimDelegate {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private InputHandler			input;
	private ABotModel				bot;
	private ICTBotControl		ctControl;
	private ITigerBotControl	tControl;
	private int						leftSpeed;
	private int						rightSpeed;
	private StringBuffer			botText;
	
	private SimPhysicsPicker	picker;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @param core
	 * @param mousePickingDelegate
	 */
	public ControlBotDelegate(ICore core) {
		super(core);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void initSimulation() {
		this.input = core.getInputHandler();
		
		picker = new SimPhysicsPicker(input, core.getDynamics(), core.getPhysicsSpace(), true) {
			
			@Override
			protected String getDevice() {
				return InputHandler.DEVICE_KEYBOARD;
			}
			

			@Override
			protected int getKey() {
				return KeyInput.KEY_INSERT;
			}
			

			@Override
			public void onAttach(DynamicPhysicsNode picked) {
				bot = ModelHelper.lookForBot(picked, 2); // The ABotModel-Node should normally be the direct parent
																		// of the "main body"
				if (bot.getBotType().equals(EBotModelType.CT_MODEL)) {
					ctControl = (ICTBotControl) bot;
				}
				else if (bot.getBotType().equals(EBotModelType.TIGER_MODEL)) {
					tControl = (ITigerBotControl) bot;
				}
			}
		};
		picker.setPickModeVisual(true);
		
		// Remove bot
		input.addAction(new InputAction() {
			@Override
			public void performAction(InputActionEvent evt) {
				bot = null;
				ctControl = null;
			}
		}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_DELETE, false);
		

		// add Actions
		input.addAction(new MyAction(250, 250), InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_UP, InputHandler.AXIS_NONE,
				false);
		input.addAction(new MyAction(0, 450), InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_LEFT, InputHandler.AXIS_NONE,
				false);
		input.addAction(new MyAction(-250, -250), InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_DOWN,
				InputHandler.AXIS_NONE, false);
		input.addAction(new MyAction(450, 0), InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_RIGHT, InputHandler.AXIS_NONE,
				false);
		

		// Kick
		input.addAction(new InputAction() {
			@Override
			public void performAction(InputActionEvent evt) {
				if (tControl != null) {
					tControl.kickStraight(100, KickMode.FORCE);
				}
			}
		}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_K, InputHandler.AXIS_NONE, false);
		
		
		// Lob
		input.addAction(new InputAction() {
			@Override
			public void performAction(InputActionEvent evt) {
				if (tControl != null) {
					tControl.lob(100, KickMode.FORCE);
				}
			}
		}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_L, InputHandler.AXIS_NONE, false);
		
		
		// Dribble
		input.addAction(new InputAction() {
			
			private boolean dribbling = false;
			
			@Override
			public void performAction(InputActionEvent evt) {
				if (!evt.getTriggerPressed()) {	// trigger released
					if (tControl == null) {
						dribbling = false;
						return;
					}

					dribbling = !dribbling;
					
					if (dribbling) {
						tControl.dribble(9000);
					} else {
						tControl.dribble(0);
					}
				}
			}
		}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_M, InputHandler.AXIS_NONE, false);
		
		
		// create SpeedLabel
		final Node botSpeedViewNode = new Node("BotView");
		Text botSpeedText = Text.createDefaultTextLabel("parameter", "");
		botSpeedText.getLocalTranslation().set(core.getDisplayWidth() - 220, core.getDisplayHeight() - 20, 0);
		botText = botSpeedText.getText();
		botSpeedViewNode.attachChild(botSpeedText);
		botSpeedViewNode.setCullHint(CullHint.Never);
		botSpeedViewNode.setRenderQueueMode(Renderer.QUEUE_ORTHO);
		botSpeedViewNode.updateGeometricState(0.0f, true);
		botSpeedViewNode.updateRenderState();
		core.getHUD().attachChild(botSpeedViewNode);
	}
	
	
	private class MyAction extends InputAction {
		
		private final int	left;
		private final int	right;
		
		
		public MyAction(int left, int right) {
			super();
			
			this.left = left;
			this.right = right;
		}
		

		@Override
		public void performAction(InputActionEvent evt) {
			if (ctControl != null) {
				
				if (evt.getTriggerPressed()) {
					leftSpeed += left;
					rightSpeed += right;
				} else {
					leftSpeed -= left;
					rightSpeed -= right;
				}
				ctControl.setSpeed(leftSpeed, rightSpeed);
				return;
			}
//			else if (tControl != null) {
//				
//				if (evt.getTriggerPressed()) {
//					leftSpeed += left;
//					rightSpeed += right;
//				} else {
//					leftSpeed -= left;
//					rightSpeed -= right;
//				}
//				tControl.getMotorControl().setSpeed(leftSpeed, leftSpeed, leftSpeed, leftSpeed);
//				return;
//			}
		}
	}
	
	
	@Override
	public void update(float interpolation, float tpc, float cps) {
		if (core.isRenderCycle()) {
			if (bot != null) {
				botText.replace(0, botText.length(), bot.getID() + ": (" + leftSpeed + " , " + rightSpeed + ")");
			} else {
				botText.replace(0, botText.length(), "");
			}
		}
	}
}
