package edu.dhbw.mannheim.tigers.sim.engine.delegates;


import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.cm;
import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.m;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.concurrent.TimeUnit;

import com.jme.animation.SpatialTransformer;
import com.jme.bounding.BoundingBox;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jme.scene.shape.Box;
import com.jme.scene.state.MaterialState;

import edu.dhbw.mannheim.tigers.sim.engine.ICore;
import edu.dhbw.mannheim.tigers.sim.engine.model.ModelHelper;
import edu.dhbw.mannheim.tigers.sim.engine.resources.TextureProvider.ETexIndex;


/**
 * Adds a rotating banner of the RoboCup-challenge 2011 to the simulation... ;-)
 * 
 * @author Gero
 */
public class TigersBannerDelegate extends ASimDelegate {
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see TigersBannerDelegate
	 * @param core
	 */
	public TigersBannerDelegate(ICore core) {
		super(core);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void initSimulation() {
		
		// TigerBanner
		Vector3f center = new Vector3f(cm(700), cm(200), cm(0));
		Vector3f shape = new Vector3f(cm(0.01f), cm(200), cm(200));
		buildTigerBanner(center, shape);
		
		center = new Vector3f(cm(-700), cm(200), cm(0));
		shape = new Vector3f(cm(0.01f), cm(200), cm(200));
		buildTigerBanner(center, shape);
		
		center = new Vector3f(cm(0), cm(200), cm(800));
		shape = new Vector3f(cm(200), cm(200), cm(0.01f));
		buildTigerBanner(center, shape);
		
		center = new Vector3f(cm(0), cm(200), cm(-800));
		shape = new Vector3f(cm(200), cm(200), cm(0.01f));
		buildTigerBanner(center, shape);
		

		// Countdown ######
		final float rotationTime = 300; // s
		
		final Node pivotNode = new Node("pivotNode");
		core.getStatics().attachChild(pivotNode);
		
		final Node rotatingNode = new Node("rotatingNode");
		pivotNode.attachChild(rotatingNode);
		
		// Build postery
		final float height = m(1.92f);
		final float heightOffset = height + cm(50);
		final float width = m(1.35f);
		final float depth = cm(1);
		
		final Box posterBox = new Box("posterBox", Vector3f.ZERO.clone(), depth, height, width);
		
		ModelHelper.applyTexture(posterBox, core.getDisplay(), ETexIndex.ROBOCUP_2011);
		
		MaterialState ms = core.getDisplay().getRenderer().createMaterialState();
		final ColorRGBA msColor = new ColorRGBA(1f, 0.7f, 0.7f, 0.7f);
		ms.setEmissive(msColor.clone());
		ms.setAmbient(msColor.clone());
		ms.setDiffuse(msColor.clone());
		posterBox.setRenderState(ms);
		
		rotatingNode.attachChild(posterBox);
		

		// Calculate time
		final Calendar robocup2010 = new GregorianCalendar(2011, 07, 04);
		final Date today = new Date();
		final long timeToRoboCup = robocup2010.getTimeInMillis() - today.getTime();
		
		final float secondsToRoboCup = TimeUnit.MILLISECONDS.toSeconds(timeToRoboCup);
		final float hoursToRoboCup = TimeUnit.MILLISECONDS.toHours(timeToRoboCup);
		

		// Rotation around pivot...
		final SpatialTransformer rotater = new SpatialTransformer(1);
		rotater.setObject(pivotNode, 0, -1);
		
		rotater.setRotation(0, 0, ModelHelper.y0.clone());
		rotater.setRotation(0, rotationTime / 4, ModelHelper.y90.clone());
		rotater.setRotation(0, rotationTime / 2, ModelHelper.y180.clone());
		rotater.setRotation(0, rotationTime * 3 / 4, ModelHelper.y270.clone());
		rotater.setRotation(0, rotationTime, ModelHelper.y360.clone());
		
		// ...finish!
		rotater.setRepeatType(SpatialTransformer.RT_WRAP);
		rotater.interpolateMissing();
		pivotNode.addController(rotater);
		

		// Move roatingNode closer to the field over the time (so it finally behaves like a spiral)
		final SpatialTransformer mover = new SpatialTransformer(1);
		mover.setObject(rotatingNode, 0, -1);
		mover.setPosition(0, 0, new Vector3f(m(hoursToRoboCup / 250), heightOffset, 0));
		mover.setPosition(0, secondsToRoboCup, new Vector3f(m(9), heightOffset, 0)); // Position to finally arrive to
		
		mover.setRepeatType(SpatialTransformer.RT_CYCLE);
		mover.interpolateMissing();
		rotatingNode.addController(mover);
		

		// Show countdown...
	}
	

	private void buildTigerBanner(Vector3f center, Vector3f shape) {
		Box simpleCube = new Box("TigerCube", center, shape.x, shape.y, shape.z);
		simpleCube.setModelBound(new BoundingBox());
		simpleCube.updateModelBound();
		ModelHelper.applyTexture(simpleCube, core.getDisplay(), ETexIndex.TIGER_ICON);
		ModelHelper.makeTransparent(simpleCube, core.getDisplay());
		core.getStatics().attachChild(simpleCube);
	}
}
