package edu.dhbw.mannheim.tigers.sim.engine.model;


import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.*;

import org.apache.log4j.Logger;

import com.jme.math.Plane;
import com.jme.math.Ray;
import com.jme.math.Vector2f;
import com.jme.math.Vector3f;
import com.jme.renderer.Camera;
import com.jme.system.DisplaySystem;
import com.jme.system.JmeException;


/**
 * This class provides some static methods supporting any operations involving the camera
 * 
 * @author Gero
 */
public class CamHelper {
	
	
	protected static final Logger	LOG	= Logger.getLogger(CamHelper.class);
	
	
	// --------------------------------------------------------------------------
	// --- supporting functions -------------------------------------------------
	// --------------------------------------------------------------------------
	public static Camera createCam(DisplaySystem display, int width, int height) {
		
		Camera newCam = null;
		
		// Create Cam
		try {
			newCam = display.getRenderer().createCamera(width, height);
		} catch (JmeException exc) {
			LOG.error("Error while creating camera!", exc);
		}
		
		// Set up the camera
		newCam.setFrustumPerspective(45.0f, ((float) width / height), cm(1), m(40));
		
		// Move the camera to a correct place and orientation.
		resetCam(newCam);
		
		return newCam;
	}
	

	public static void resetCam(Camera cam) {
		
		cam.setFrame(getStartLocation(), getLeftAxis(), getUpAxis(), getStartDirection());
		cam.update();
	}
	

	private static Vector3f getLeftAxis() {
		return new Vector3f(0, 0, 1f); // Z
	}
	

	private static Vector3f getUpAxis() {
		return new Vector3f(Vector3f.UNIT_Y); // Y
	}
	

	private static Vector3f getStartDirection() {
		return new Vector3f(-1f, 0, 0); // -X
	}
	

	private static Vector3f getStartLocation() {
		return new Vector3f(m(1.5f), m(0.75f), 0);
	}
	
	
	/**
	 * Calculates the intersection-point between the line (ray) "Camera-Mouse" and the y0-plane
	 * 
	 * @return A new {@link Vector3f} containing this information
	 * @see #ray
	 */
	public static Vector3f calculateMouseOnGround(Vector2f mousePosition) {
		Ray ray = new Ray();
		Plane y0Plane = new Plane(Vector3f.UNIT_Y.clone(), 0);
		
		// Get vector and position offset of the ray
		DisplaySystem.getDisplaySystem().getWorldCoordinates(mousePosition, 0, ray.getOrigin());
		DisplaySystem.getDisplaySystem().getWorldCoordinates(mousePosition, 0.3f, ray.getDirection());
		ray.getDirection().subtractLocal(ray.getOrigin()).normalizeLocal();
		
		// Plane is y0
		Vector3f intersection = new Vector3f();
		ray.intersectsWherePlane(y0Plane, intersection);
		
		return intersection;
	}
}
