package edu.dhbw.mannheim.tigers.sim.engine.model;

import com.jme.math.Quaternion;
import com.jme.math.Vector3f;
import com.jme.scene.Node;


/**
 * Provides access to some basic model-control methods
 * 
 * @author Gero
 */
public interface IBasicModelControl {
	
	/**
	 * @return changed
	 */
	public boolean setActive(boolean active);
	

	/**
	 * A hard reset of the model's location. Calls {@link #setLocation(Vector3f, boolean, boolean, boolean)} with
	 * <code> true, true, true</code>.
	 */
	public void setLocation(Vector3f newPosition);
	

	/**
	 * A hard reset of the model's location. Clears speed, torque and velocities if wished.
	 * 
	 * @param newPosition
	 * @param clearForce
	 * @param clearTorque
	 * @param clearVelocities
	 */
	public void setLocation(Vector3f newPosition, boolean clearForce, boolean clearTorque, boolean clearVelocities);
	

	/**
	 * A hard reset of the model's rotation. Calls {@link #setRotation(Vector3f, boolean, boolean, boolean)} with
	 * <code> true, true, true</code>.
	 */
	public void setRotation(Quaternion quaternion);
	

	/**
	 * A hard reset of the model's rotation. Clears speed, torque and velocities if wished.
	 * 
	 * @param quaternion
	 * @param clearForce
	 * @param clearTorque
	 * @param clearVelocities
	 */
	public void setRotation(Quaternion quaternion, boolean clearForce, boolean clearTorque, boolean clearVelocities);
	

	/**
	 * @return the "root" node of the model
	 */
	public Node getNode();
	

	/**
	 * @return The bots height (In 'model.xml' dimensions, so you need to explicitly use
	 *         {@link UnitConversion#data(float)} to convert to simulation dimensions!!!)
	 */
	public abstract float getHeight();
	

	/**
	 * @return The bots radius (In 'model.xml' dimensions, so you need to explicitly use
	 *         {@link UnitConversion#data(float)} to convert to simulation dimensions!!!)
	 */
	public abstract float getRadius();
}
