package edu.dhbw.mannheim.tigers.sim.engine.model;


import java.io.IOException;

import com.jme.scene.Node;
import com.jme.util.export.JMEExporter;
import com.jme.util.export.JMEImporter;
import com.jme.util.export.Savable;


/**
 * This is the base class for any sim user-data for tagging the scenegraph-objects with valueable information.
 * 
 * {@link Node#setUserData(String, Savable)}
 * 
 * @author Gero
 */
public class UserData implements Savable {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	public static final String	TYPE		= "type";
	public static final String	PICKABLE	= "pickable";
	
	public enum Value {
		GROUND,
		BOT,
		BALL,
		TRUE;
	}
	
	private final Value	value;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public UserData(Value value) {
		this.value = value;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public Class<?> getClassTag() {
		return UserData.class;
	}
	

	@Override
	public void read(JMEImporter im) throws IOException {
		throw new UnsupportedOperationException();
	}
	

	@Override
	public void write(JMEExporter ex) throws IOException {
		throw new UnsupportedOperationException();
	}
	

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		
		if (obj instanceof Value) {
			return value.equals(obj);
		} else if (obj instanceof UserData) {
			return value.equals(((UserData) obj).getValue());
		} else {
			return false;
		}
	}
	

	@Override
	public int hashCode() {
		return value.hashCode();
	}
	

	public Value getValue() {
		return value;
	}
}
