package edu.dhbw.mannheim.tigers.sim.engine.model.bot.parts;

import java.util.Iterator;
import java.util.LinkedList;

import com.jme.intersection.PickData;
import com.jme.intersection.TrianglePickResults;
import com.jme.math.Ray;
import com.jme.math.Vector3f;
import com.jme.scene.Node;
import com.jme.scene.Spatial;
import com.jme.util.export.Savable;
import com.jmex.physics.DynamicPhysicsNode;
import com.jmex.physics.PhysicsNode;

import edu.dhbw.mannheim.tigers.sim.engine.model.IUpdateableModel;
import edu.dhbw.mannheim.tigers.sim.engine.model.UserData;
import edu.dhbw.mannheim.tigers.sim.engine.model.UserData.Value;


/**
 * This class simulates a simple IR-sensor which can detect whether a ball is in its way
 * 
 * @author Gero
 */
public class IRSensor implements IBotModelPart {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Config
	private final Node							irNode;
	private final Node							irNode2;
	private final Vector3f						offset;
	private final Vector3f						direction;
	private final float							delay;
	
//	// Physics-picker
//	private final PhysicsRay					irRay;
//	private final InputHandler					detectionHandler;
//	private final PhysicsSpace					space;
//	private PhysicsCollisionGeometry			nearestPickedGeom	= null;

	// Visual-picker
	private final Ray								pickRay			= new Ray();
	private final TrianglePickResults		pickResults		= new TrianglePickResults();
	private final Node							branch;
	private final float							maxDistance;
	
	/** Together with {@link StateChange}s these queue is necessary for proper delay of the sensor-states */
	private final LinkedList<StateChange>	stateChanges	= new LinkedList<StateChange>();
	private boolean								isBlocked		= false;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see IRSensor
	 * @param node The node to whom this sensor should be attached to
	 * @param offset Local translation
	 * @param direction Local direction of the ray [mm]
	 * @param branch The branch of the scene-graph to check for collisions
	 * @param delay How long it takes until the sensor reacts (min) [s]
	 */
	public IRSensor(PhysicsNode node, Vector3f offset, Vector3f direction, Node branch, float delay) {
		irNode = new Node("irNode");
		node.attachChild(irNode);
		
		irNode2 = new Node("irNode2");
		node.attachChild(irNode2);
		
		maxDistance = direction.length() * 0.001f;
		
		
		this.offset = new Vector3f(offset);
		irNode.setLocalTranslation(this.offset);
		
		this.direction = new Vector3f(direction);
		
		final Vector3f vec2 = new Vector3f(this.offset);
		vec2.addLocal(this.direction);
		irNode2.setLocalTranslation(vec2);
		
		this.delay = delay;
		
		this.branch = branch;
		
//		space = irNode.getSpace();
//		irRay = irNode.createRay("irRay");
//		
//		detectionHandler = new InputHandler();
//		detectionHandler.addAction(new PhysicsPickAction(), irRay.getCollisionEventHandler(), false);
	}
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void doUpdate(long cycleNumber, float tpc) {
//		// Update ray position/direction
//		irRay.getLocalTranslation().set(irNode.getWorldTranslation()).addLocal(offset);
//		irRay.getLocalScale().set(direction);
//		irRay.updateWorldVectors();
//		
//		// Perform pick
//		detectionHandler.setEnabled(true);
//		space.pick(irRay);
//		detectionHandler.update(0);
//		detectionHandler.setEnabled(false);
		
		
//		// System.out.println("#####");
//		
//		// Check the eventually picked geometry
//		if (nearestPickedGeom != null && !nearestPickedGeom.getPhysicsNode().isStatic()) {
//			DynamicPhysicsNode pickedNode = (DynamicPhysicsNode) nearestPickedGeom.getPhysicsNode();
//			// System.out.println("Picked: " + pickedNode);
//			if (pickedNode != null && isBall(pickedNode)) {
//				
//				// There really is a ball!
//				switchStateDelayed(true);
////				System.out.println("Ball detected!");
//			} else {
//				switchStateDelayed(false);
//			}
//		} else {
//			switchStateDelayed(false);
//		}
//		
//		nearestPickedGeom = null;
		
		// Update ray position/direction
		irNode.updateWorldVectors();
		irNode2.updateWorldVectors();
		
		pickRay.origin.set(irNode.getWorldTranslation());
		irNode2.getWorldTranslation().subtract(pickRay.origin, pickRay.direction);
		pickRay.direction.normalizeLocal();
		
		pickResults.clear();
		pickResults.setCheckDistance(true);
		
//		// Debug
//		if (curve != null) {
//			curve.removeFromParent();
//		}
//		
//		curve = ModelHelper.draw2DLine(irNode.getParent(), offset, offset.add(direction), ColorRGBA.cyan);
		
		
		// Pick!
		branch.findPick(pickRay, pickResults);
		
		
		// Step through results
		if (pickResults.getNumber() == 0) {
			switchStateDelayed(false);
		} else {
			loopResults: for (int i = 0; i < pickResults.getNumber(); i++) {
				PickData data = pickResults.getPickData(i);
				if (data.getTargetTris() != null) {
					Spatial target = data.getTargetMesh();
					while (target != null) {
						if (target instanceof DynamicPhysicsNode) {
							// Found something!
							DynamicPhysicsNode pickedNode = (DynamicPhysicsNode) target;	
							
							final float distance = pickedNode.getWorldTranslation().distance(pickRay.origin);
							
							if (isBall(pickedNode) && distance < maxDistance) {
								// There really is a ball!
								switchStateDelayed(true);
//								System.out.println("Ball detected!");
							} else {
								switchStateDelayed(false);
							}
							
							break loopResults;
						} else {
							switchStateDelayed(false);
						}
						target = target.getParent();
					}
				} else {
					switchStateDelayed(false);
				}
			}
		}
		
		
		// Check for changes of state
		Iterator<StateChange> it = stateChanges.iterator();
		while (it.hasNext()) {
			StateChange c = it.next();
			c.update(cycleNumber, tpc);
			if (c.isDelayed()) {
				// If the StateChange is properly delayed, apply and remove it.
//				System.out.println("Ball applied!");
				isBlocked = c.blocked;
				it.remove();
			}
		}
	}
	

	/**
	 * Enqueues a {@link StateChange} in the {@link #stateChanges} if necessary
	 * @param newState
	 */
	private void switchStateDelayed(boolean newState) {
		if (stateChanges.isEmpty()) {
			if (newState != isBlocked) {
				stateChanges.add(new StateChange(newState));
			}
		} else {
			if (stateChanges.getLast().blocked != newState) {
				stateChanges.add(new StateChange(newState));
			}
		}
	}
	

	private boolean isBall(DynamicPhysicsNode pickedNode) {
		
		boolean result = false;
		Savable sav = pickedNode.getUserData(UserData.TYPE);
		if (sav instanceof UserData) {
			UserData ud = (UserData) sav;
			result = ud.getValue().equals(Value.BALL);
		}
		
		return result;
	}
	
	
//	private class PhysicsPickAction extends InputAction {
//		public void performAction(InputActionEvent evt) {
//			
//			final ContactInfo info = (ContactInfo) evt.getTriggerData();
//			PhysicsCollisionGeometry other = info.getGeometry1() == irRay ? info.getGeometry2() : info.getGeometry1();
//			nearestPickedGeom = other;
//		}
//	}
	
	
	/**
	 * @return Whether the IR-ray is blocked (by the ball)
	 */
	public boolean isBlocked() {
		return isBlocked;
	}
	
	
	/**
	 * This class represents a change of the IR-sensors state which has to be delayed before it is applied
	 */
	private class StateChange implements IUpdateableModel {
		
		private final boolean	blocked;
		private float				timeSinceChange	= 0;
		
		
		public StateChange(boolean blocked) {
			this.blocked = blocked;
		}
		

		@Override
		public void update(long cycleNumber, float tpc) {
			timeSinceChange += tpc;
		}
		

		boolean isDelayed() {
			return timeSinceChange >= delay;
		}
	}
}
