package edu.dhbw.mannheim.tigers.sim.engine.model.bot.tiger;

import com.jmex.physics.RotationalJointAxis;

import edu.dhbw.mannheim.tigers.sim.engine.model.bot.parts.APid;


/**
 * {@link APid}-implementation for the {@link TigerMotorController}-motors
 * 
 * @author Gero
 */
public class MotorPid extends APid {
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private final RotationalJointAxis	motor;
	/** [ns], delta t between two calls of {@link #getFeedback()} */
	private final double						dtExpected;
	
	private static final long				FIRST_CALL			= -1;
	private long								lastFeedbackCall	= FIRST_CALL;
	
	private float lastPosition = 0;

	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see MotorPid
	  * @param motor
	  * @param cps
	  */
	public MotorPid(RotationalJointAxis motor, int cps) {
		super();
		
		this.motor = motor;
		this.dtExpected = (1f / cps) * 1e9;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected float getFeedback() {
		
		/*
		int32_t dt_real = (xTaskGetTickCount() - pData->lastTickCount)*10*portTICK_RATE_MS;

		int32_t n_enc = (int16_t)pData->pTIM->CNT;
		pData->pTIM->CNT = 0;

		if(n_enc - pData->lastEncCount > 1000)
			n_enc = pData->lastEncCount + 1000;
		else if(pData->lastEncCount - n_enc > 1000)
			n_enc = pData->lastEncCount - 1000;

		pData->lastEncCount = n_enc;

		n_enc = (n_enc * 13 * pData->pid.dt)/dt_real;

		pData->lastTickCount = xTaskGetTickCount();

		return -n_enc;	// invert feedback due to transmission
		*/

		// Timing
		final long now = System.nanoTime();
		long dtReal = 1;
		if (lastFeedbackCall != FIRST_CALL) {
			dtReal = now - lastFeedbackCall;
		}
		lastFeedbackCall = now;
		
		double posDiff = motor.getPosition()-lastPosition;
		lastPosition = motor.getPosition();
		if(posDiff < -Math.PI)
		{
			posDiff = Math.PI*2 + posDiff;
		}
		if(posDiff > Math.PI)
		{
			posDiff = posDiff - Math.PI*2;
		}
		
		final double curRot = posDiff;
		
		final double curTicks = curRot * 1440;				// [ticks/s]
		// Fake: Assume the current speed represents the average speed during the last cycle
		final double ticks = curTicks * 2.5 * dtExpected / dtReal;	// [ticks]
		
		return (float) ticks;
	}
}
