package edu.dhbw.mannheim.tigers.sim.engine.modes.match;

import java.util.List;
import java.util.concurrent.Future;

import com.jme.renderer.ColorRGBA;

import edu.dhbw.mannheim.tigers.sim.engine.com.ERoboCupTeam;
import edu.dhbw.mannheim.tigers.sim.engine.model.ball.IBallModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.BotID;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBasicBotModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.modes.IMatchControlProvider;
import edu.dhbw.mannheim.tigers.sim.engine.modes.IVisDataInterpreterProvider;


/**
 * This interface encapsulates controlling methods which can modify a running RoboCup-match
 * 
 * @author Gero
 */
public interface IMatchModeControl extends IMatchControlProvider, IVisDataInterpreterProvider {
	
	/**
	 * Adds a bot-model of the given type to the simulation
	 * 
	 * @param count
	 * @param type
	 * @param color
	 * @param team
	 * @param left Whether the models are added along the -Z or Z-axis from the field-center 
	 * @return The {@link Future} that contains the task
	 */
	public Future<List<IBasicBotModelControl>> addBotModelToSimulation(final int count, final String type,
			final ColorRGBA color, final ERoboCupTeam team, final boolean left);
	

	/**
	 * @param id
	 * @return The {@link Future} with the task
	 */
	public Future<Boolean> removeBotModel(BotID... id);
	

	/**
	 * @return  The {@link Future} with the task
	 */
	public Future<IBallModelControl> addBallToSimulation();
	

	/**
	 * @return The {@link Future} with the task
	 */
	public Future<Boolean> removeBall();
}
