package edu.dhbw.mannheim.tigers.sim.engine.plugins;

import org.apache.commons.configuration.SubnodeConfiguration;


/**
 * Base class for any plugin
 * 
 * @author Gero
 */
public abstract class ASimPlugin {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	protected final SubnodeConfiguration	config;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see ASimPlugin
	 * @param config
	 */
	public ASimPlugin(SubnodeConfiguration config) {
		checkConfigConsistency(config);
		this.config = config;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return The unique {@link PluginID}
	 */
	public abstract PluginID getID();
	

	/**
	 * @return The {@link EPluginType}
	 */
	public abstract EPluginType getType();
	

	/**
	 * Check the given {@link SubnodeConfiguration} for consistency (in terms of needed settings for this
	 * {@link ASimPlugin}-implementation)
	 * <p>
	 * Example:
	 * <pre>
	 *if (config.getProperty("properties.overlapx") == null) {
	 *	config.setProperty("properties.overlapx", DEFAULT_OVERLAP_X);
	 *}
	 * </pre>
	 * </p>
	 * 
	 * @param config
	 */
	public abstract void checkConfigConsistency(SubnodeConfiguration config);
	

	/**
	 * @return The {@link SubnodeConfiguration}-object that represents the loaded configuration for this
	 *         {@link ASimPlugin}
	 */
	public SubnodeConfiguration getConfig() {
		checkConfigConsistency(config);
		return config;
	}
	

	@Override
	public String toString() {
		return "[" + getType() + " " + getID() + "]";
	}
}
