package edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization;

import java.util.List;

import com.jme.scene.Node;
import com.jme.system.DisplaySystem;

import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBasicBotModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.modes.IMatchControlProvider;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.Poi;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.PoiList;


/**
 * Default-implementation of {@link APOIDrawer} that draws bots.
 * 
 * @author Gero
 */
public class DefaultBotDrawer extends APOIDrawer {
	
	private final IMatchControlProvider	matchControl;
	
	
	/**
	 * @param root
	 * @param renderQueueMode
	 * @param display
	 * @param matchControl
	 */
	public DefaultBotDrawer(Node root, int renderQueueMode, DisplaySystem display, IMatchControlProvider matchControl) {
		super(root, renderQueueMode, display);
		
		this.matchControl = matchControl;
	}
	

	@Override
	public void displayInfo(SimVisData data) {
		doDisplayInfo(data.getPredictionsList());
		attachInfoText(data);
	}
	

	@Override
	protected Integer identify(PoiList info) {
		Integer id = info.getId();
		
		List<Poi> pois = info.getPoisList();
		if (!pois.isEmpty()) {
			Poi poi = pois.get(0);
			if (poi.getValue() < 0) {
				id = -id;
			}
		}
		
		return id;
	}
	

	@Override
	protected AVisualObject createSinglePoi(Poi poi) {
		return new PredictedBot(poi, localRoot, display);
	}
	

	private void attachInfoText(SimVisData msg) {
		List<? extends IBasicBotModelControl> botModelControls = matchControl.getBotControls();
		
		for (PoiList prediction : msg.getPredictionsList()) {
			if (!prediction.getPoisList().isEmpty()) {
				final Poi poi = prediction.getPois(0);
				float teamVal = poi.getValue();
				
				if (teamVal > 0) {	// yellow; our team!
					for (IBasicBotModelControl control : botModelControls) {
						final int predictionID = prediction.getId();
						if (control.getID().getIntValue() == predictionID && !poi.getTextList().isEmpty()) {
							control.setInfoText(poi.getText(0));
							break;
						}
					}
				}
			}
		}
	}
}
