package edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization;

import java.util.List;

import com.jme.scene.Node;
import com.jme.system.DisplaySystem;

import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.Poi;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.PoiList;


/**
 * Implementation of {@link APOIDrawer} that draws paths.
 * 
 * @author Gero
 */
public class DefaultPathDrawer extends APOIDrawer {
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @param root
	 * @param renderQueueMode
	 * @param display
	 * @param type
	 */
	public DefaultPathDrawer(Node root, int renderQueueMode, DisplaySystem display) {
		super(root, renderQueueMode, display);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void displayInfo(SimVisData data) {
		doDisplayInfo(data.getPathsList());
	}
	

	@Override
	protected void drawInfo(List<Poi> pois, List<AVisualObject> store) {
		
		if (pois.size() < 2) { // Need at least two points to draw a line!
			return;
		}
		
		undrawInfo(store); // Do _not reuse old lines; but delete them when new one comes in!!
		store.clear();
		
		// log.debug("### New Line ");
		Poi last = pois.get(0);
		for (int i = 1; i < pois.size(); i++) {
			Poi current = pois.get(i);
			
			AVisualObject visual = new PlanedPath(last, current, localRoot);
			visual.updateRenderState();
			store.add(visual);
			
			last = current;
		}
	}
   
   
   @Override
   protected Integer identify(PoiList info) {
   	Integer id = info.getId();
   	
   	List<Poi> pois = info.getPoisList();
   	if (!pois.isEmpty()) {
   		Poi poi = pois.get(0);
   		if (poi.getValue() < 0) {
   			id = -id;
   		}
   	}
   	
   	return id;
   }
	

	@Override
	protected AVisualObject createSinglePoi(Poi poi) {
		/* Not used */
		return null;
	}
}
