package edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization;

import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.cm;

import com.jme.curve.PolylineCurve;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;

import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.Poi;


/**
 * Visual representation of a planned bot-path
 * 
 * @author Gero
 */
public class PlanedPath extends AVisualObject {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Constants
	private final ColorRGBA			color						= ColorRGBA.red.clone();
	private static final float		PATH_HEIGHT_OFFSET	= cm(2.5f);
	
	// Visuals
	private final PolylineCurve	curve;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see PlanedPath
	  * @param poi
	  * @param poi2
	  * @param parent
	  */
	PlanedPath(Poi poi, Poi poi2, Node parent) {
		super(poi, parent);
		
		curve = draw2DLine(poi.getPos(), poi2.getPos(), PATH_HEIGHT_OFFSET, color);
		localRoot.attachChild(curve);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected AVisualObject drawSingle(Poi poi) {
		/* Does nothing */
		return this;
	}
	

	@Override
	protected void undraw() {
		curve.removeFromParent();
	}
	

	@Override
	protected void updateRenderState() {
		curve.updateRenderState();
	}
}
