package edu.dhbw.mannheim.tigers.sim.engine.resources;

import java.awt.Image;
import java.util.HashMap;

import javax.swing.ImageIcon;

import com.jme.image.Texture;
import com.jme.image.Texture.MagnificationFilter;
import com.jme.image.Texture.MinificationFilter;
import com.jme.util.TextureManager;

import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.resources.AResourceProvider;


/**
 * Provides textures to the model.
 * Must get initialized by calling initialize() before use, else SimModelException is thrown.
 * 
 * @author Gero
 */
public class TextureProvider extends AResourceProvider<TextureProvider.ETexIndex, Image> implements ITextureProvider {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final String		TEXTURES	= "resources/textures/";
	
	private static ITextureProvider	instance	= null;
	
	
	/**
	 * One enum-constant represents a texture which can be loaded from this {@link ITextureProvider}-implementation
	 */
	public enum ETexIndex {
		SIMPLE_FIELD,
		REFLECTOR,
		TIGER_ICON,
		TIGER_TRAC_LOGO,
		DHBW_HEADER,
		ROBOCUP_2011,
		SKYBOX_NORTH;
	}
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	private TextureProvider() {
		super(TextureProvider.class);
	}
	

	public static synchronized ITextureProvider getInstance() {
		if (instance == null) {
			instance = new TextureProvider();
		}
		return instance;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void doInitialize() {
		storage = new HashMap<ETexIndex, Image>();
	}
	

	@Override
	protected void doLoad() {
		
		storage.put(ETexIndex.SIMPLE_FIELD, new ImageIcon(getPath("simpleField.png")).getImage());
		
		storage.put(ETexIndex.REFLECTOR, new ImageIcon(getPath("reflector.jpg")).getImage());
		
		storage.put(ETexIndex.TIGER_ICON, new ImageIcon(getPath("tigerIcon.png")).getImage());
		
		storage.put(ETexIndex.DHBW_HEADER, new ImageIcon(getPath("dhbw_header.jpg")).getImage());
		
		storage.put(ETexIndex.TIGER_TRAC_LOGO, new ImageIcon(getPath("tigerTracLogo.jpg")).getImage());
		
		storage.put(ETexIndex.ROBOCUP_2011, new ImageIcon(getPath("robocup_2011.jpg")).getImage());
		
		storage.put(ETexIndex.SKYBOX_NORTH, new ImageIcon(getPath("skybox_north.jpg")).getImage());
	}
	

	private String getPath(String filename) {
		return TEXTURES + filename;
	}
	

	@Override
	public Texture getTexture(ETexIndex key) throws SimResourceException {
		
		final Image img = getData(key);
		
		final Texture result = TextureManager.loadTexture(img, MinificationFilter.BilinearNearestMipMap,
				MagnificationFilter.Bilinear, true);
		
		return result;
	}
	

	@Override
	public Image getImage(ETexIndex key) throws SimResourceException {
		return getData(key);
	}
}
