package edu.dhbw.mannheim.tigers.sim.gui.swing;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;

import com.jme.math.Vector2f;
import com.jmex.physics.contact.MutableContactInfo;
import com.jmex.physics.material.Material;

import edu.dhbw.mannheim.tigers.sim.engine.model.material.RoboCupMaterial;


/**
 * This frame is for debugging/development only
 * 
 * @author Gero
 * 
 */
public class ParameterFrame extends JFrame {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long	serialVersionUID	= 1713478236489329037L;
	

	private final JComboBox		leftMatSwitch;
	private final JComboBox		rightMatSwitch;
	private final JButton		applyBtn				= new JButton("Apply");
	
	private final JPanel			propPanel			= new JPanel();
	
	private final JLabel			bounceLabel			= new JLabel("Bounce: ");
	private final JTextField	bounceText			= new JTextField(30);
	
	private final JLabel			dampingLabel		= new JLabel("Damping: ");
	private final JTextField	dampingText			= new JTextField(30);
	
	private final JLabel			minBounceLabel		= new JLabel("min Bounce: ");
	private final JTextField	minBounceText		= new JTextField(30);
	
	private final JLabel			muLabel				= new JLabel("Mu: ");
	private final JTextField	muText				= new JTextField(30);
	
	private final JLabel			muOrthogonalLabel	= new JLabel("Mu orthogonal: ");
	private final JTextField	muOrthogonalText	= new JTextField(30);
	
	private final JLabel			springLabel			= new JLabel("Spring: ");
	private final JTextField	springText			= new JTextField(30);
	
	private final JLabel			slipXLabel			= new JLabel("Slip X: ");
	private final JTextField	slipXText			= new JTextField(30);
	
	private final JLabel			slipYLabel			= new JLabel("Slip Y: ");
	private final JTextField	slipYText			= new JTextField(30);
	

	private Material				leftMat				= null;
	private Material				rightMat				= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see ParameterFrame
	 */
	public ParameterFrame() {
		// Set LookAndFeel to: OS Default, looks and feels best ;-)
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception err) {
		}
		
		setLayout(new BorderLayout());
		
		// Material switch
		Collection<Material> materials = RoboCupMaterial.getMaterials();
		Material[] materialArray = materials.toArray(new Material[materials.size()]);
		
		leftMatSwitch = new JComboBox(materialArray);
		leftMatSwitch.setSelectedItem(RoboCupMaterial.BOT_DRIBBLER);
		this.add(leftMatSwitch, BorderLayout.WEST);
		
		rightMatSwitch = new JComboBox(materialArray);
		rightMatSwitch.setSelectedItem(RoboCupMaterial.BALL);
		this.add(rightMatSwitch, BorderLayout.EAST);
		
		ActionListener al = null;
		leftMatSwitch.addActionListener(al = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				leftMat = (Material) leftMatSwitch.getSelectedItem();
				leftMatSwitch.repaint();
				
				rightMat = (Material) rightMatSwitch.getSelectedItem();
				rightMatSwitch.repaint();
				materialToProperties(leftMat, rightMat);
			}
		});
		rightMatSwitch.addActionListener(al);
		

		// Apply button
		applyBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent paramActionEvent) {
				MutableContactInfo info = propertiesToMaterial();
				
				if (leftMat != null) {
					leftMat.putContactHandlingDetails(rightMat, info);
				}
				if (rightMat != null) {
					rightMat.putContactHandlingDetails(leftMat, info);
				}
				
				// If cleanF has changed anything, this will display it
				materialToProperties(leftMat, rightMat);
			}
		});
		this.add(applyBtn, BorderLayout.SOUTH);
		

		propPanel.setBorder(BorderFactory.createTitledBorder("Parameter"));
		propPanel.setLayout(new GridLayout(9, 2));
		
		propPanel.add(bounceLabel);
		propPanel.add(bounceText);
		
		propPanel.add(dampingLabel);
		propPanel.add(dampingText);
		
		propPanel.add(minBounceLabel);
		propPanel.add(minBounceText);
		
		propPanel.add(muLabel);
		propPanel.add(muText);
		
		propPanel.add(muOrthogonalLabel);
		propPanel.add(muOrthogonalText);
		
		propPanel.add(springLabel);
		propPanel.add(springText);
		
		propPanel.add(slipXLabel);
		propPanel.add(slipXText);
		
		propPanel.add(slipYLabel);
		propPanel.add(slipYText);
		this.add(propPanel);
		
		// Initialize
		al.actionPerformed(null);
		
		this.setSize(400, 300);
		setVisible(true);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	private MutableContactInfo propertiesToMaterial() {
		MutableContactInfo info = new MutableContactInfo();
		info.setBounce(cleanF(bounceText.getText()));
		info.setDampingCoefficient(cleanF(dampingText.getText()));
		info.setMinimumBounceVelocity(cleanF(minBounceText.getText()));
		info.setMu(cleanF(muText.getText()));
		info.setMuOrthogonal(cleanF(muOrthogonalText.getText()));
		info.setSpringConstant(cleanF(springText.getText()));
		float slipX = cleanF(slipXText.getText());
		float slipY = cleanF(slipYText.getText());
		info.setSlip(new Vector2f(slipX, slipY));
		return info;
	}
	
	private Vector2f	temp	= new Vector2f();
	
	
	private void materialToProperties(Material left, Material right) {
		
		MutableContactInfo info;
		if (left == null) {
			info = (MutableContactInfo) right.getContactHandlingDetails(left);
		} else {
			info = (MutableContactInfo) left.getContactHandlingDetails(right);
			if (info == null) {
				info = new MutableContactInfo();
			}
		}
		
		bounceText.setText(info.getBounce() + "");
		dampingText.setText(info.getDampingCoefficient() + "");
		minBounceText.setText(info.getMinimumBounceVelocity() + "");
		muText.setText(info.getMu() + "");
		muOrthogonalText.setText(info.getMuOrthogonal() + "");
		springText.setText(info.getSpringConstant() + "");
		slipXText.setText(info.getSlip(temp).x + "");
		slipYText.setText(info.getSlip(temp).y + "");
	}
	

	private float cleanF(String str) {
		return Float.valueOf(str);
	}
}
