package edu.dhbw.mannheim.tigers.sim.gui.swing;


import javax.swing.JOptionPane;

import org.apache.log4j.Appender;
import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ESimEngineMode;
import edu.dhbw.mannheim.tigers.sim.gui.IGUI;
import edu.dhbw.mannheim.tigers.sim.settings.ISettingsProvider;
import edu.dhbw.mannheim.tigers.sim.settings.data.IApplicationProperties;
import edu.dhbw.mannheim.tigers.sim.util.logging.StartupLogAppender;


/**
 * This is a Swing-implementation of the {@link IGUI}-interface, which handles the implementation
 * between GUI and controller.
 * 
 * @author Gero
 */
public class SwingGUI implements IGUI {
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	protected final Logger				log	= Logger.getLogger(getClass());
	
	private final IInputController	inputController;
	
	private final SimMainWindow		mainWindow;
//	private ParameterFrame				parameterFrame;
	
	private Appender						startupLogger;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see SwingGUI
	 * @param inputController
	 */
	public SwingGUI(IInputController inputController) {
		
		this.inputController = inputController;
		
		mainWindow = new SimMainWindow(this);
//		parameterFrame = new ParameterFrame();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void intialize(Appender logAppender) {
		this.startupLogger = logAppender;
		mainWindow.init();
	}
	

	@Override
	public void refreshEngineState(ESimEngineState newState) {
		mainWindow.refreshEngineState(newState);
	}
	

	@Override
	public void setEnabled(boolean enabled) {
		mainWindow.setEnabled(enabled);
	}
	

	@Override
	public IInputController getInputController() {
		return inputController;
	}
	

	@Override
	public ESimEngineMode getSelectedMode() {
		return mainWindow.getEngineControl().getSelectedMode();
	}
	

	@Override
	public void close() {
		
		IApplicationProperties props = ISettingsProvider.INSTANCE.getSimProperties().getApplicationProperties();
		props.setMainWindowPosX(mainWindow.getX());
		props.setMainWindowPosY(mainWindow.getY());
		props.setMainWindowHeight(mainWindow.getHeight());
		props.setMainWindowWidth(mainWindow.getWidth());
		
		mainWindow.setEnabled(false);
		mainWindow.setVisible(false);
		mainWindow.close();
		mainWindow.dispose();
		
//		parameterFrame.setEnabled(false);
//		parameterFrame.setVisible(false);
//		parameterFrame.dispose();
	}
	

	@Override
	public boolean verifyEndOfSimulation() {
		
		return JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(null, "Really want to stop the simulation?",
				"Stupid Question...", JOptionPane.OK_CANCEL_OPTION);
	}
	

	public StartupLogAppender getStartupLogAppender() {
		return (StartupLogAppender) this.startupLogger;
	}
}
