package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Calls {@link IInputController#pause()} pauses the whole simulation
 * 
 * @author Gero
 */
public class SimPauseFullAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long				serialVersionUID	= 297289761313620418L;
	
	private static final String			TITLE					= "Pause";
	private static final String			DESCRIPTION			= "This pauses the complete simulation, including camera etc.";
	
	private static SimPauseFullAction	instance;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public synchronized static SimPauseFullAction getInstance(SwingGUI gui) {
		if (instance == null) {
			EImage imgType = EImage.CONTROL_PAUSE;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new SimPauseFullAction(gui, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(SimPauseFullAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new SimPauseFullAction(gui);
			}
		}
		return instance;
	}
	

	private SimPauseFullAction(SwingGUI gui) {
		super(TITLE, DESCRIPTION, gui.getInputController());
	}
	

	private SimPauseFullAction(SwingGUI gui, Icon icon) {
		super(TITLE, DESCRIPTION, gui.getInputController(), icon);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		
		input.pause();
	}
}
