package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Calls {@link IInputController#stop()}
 * 
 * @author Gero
 */
public class SimStopAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long		serialVersionUID	= 3045662841305010742L;
	
	private static final String	TITLE					= "Stop";
	private static String			DESCRIPTION			= "This stops the whole simulation that it could NOT be resumed";
	
	private static SimStopAction	instance;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public synchronized static SimStopAction getInstance(SwingGUI gui) {
		if (instance == null) {
			EImage imgType = EImage.CONTROL_STOP;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new SimStopAction(gui, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(SimPauseFullAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new SimStopAction(gui);
			}
		}
		return instance;
	}
	

	private SimStopAction(SwingGUI gui) {
		super(TITLE, DESCRIPTION, gui.getInputController());
	}
	

	private SimStopAction(SwingGUI gui, Icon icon) {
		super(TITLE, DESCRIPTION, gui.getInputController(), icon);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		input.stop();
	}
}
