package edu.dhbw.mannheim.tigers.sim.gui.swing.settings;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;

import net.miginfocom.swing.MigLayout;


/**
 * Base class for sim settings-window
 * 
 * @author Malte, Gero
 */
public abstract class ASettingsFrame extends JFrame {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long					serialVersionUID	= -4807938894305160676L;
	

	public static final Dimension				BTN_SIZE				= new Dimension(75, 23);
	
	protected final SwingGUI					gui;
	
	protected final List<ASettingsPanel>	settingsPanels		= new ArrayList<ASettingsPanel>();
	
	private final JPanel							buttonsPanel;
	
	private final JButton						ok;
	private final ActionListener				okListener			= new OkListener();
	private final JButton						cancel;
	private final ActionListener				cancelListener		= new CancelListener();
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see ASettingsFrame
	 * @param gui
	 */
	public ASettingsFrame(final SwingGUI gui) {
		super("Settings");
		
		this.gui = gui;
		

		// General setup
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				cancelListener.actionPerformed(null);
			}
		});
		this.setLayout(new MigLayout("wrap 1, insets 5", "[grow, fill]", ""));
		
		// "Escape" to cancel
		this.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
					onCancel();
				}
			}
		});
		

		// Tabs
		JTabbedPane tabbedPane = new JTabbedPane();
		
		addSettingsPanels(settingsPanels, gui);
		
		// Add tabs!
		for (ASettingsPanel panel : settingsPanels) {
			tabbedPane.addTab(panel.getTitle(), panel);
		}
		
		// Initialize the tabs (except EngineSettingsPanel, which needs some special treatment...)
		for (ASettingsPanel panel : settingsPanels) {
			panel.initialize();
		}
		
		this.add(tabbedPane, BorderLayout.CENTER);
		

		// Buttons
		buttonsPanel = new JPanel();
		ok = new JButton("Save");
		ok.addActionListener(okListener);
		sizeBtn(ok, BTN_SIZE);
		buttonsPanel.add(ok);
		
		cancel = new JButton("Cancel");
		cancel.addActionListener(cancelListener);
		sizeBtn(cancel, BTN_SIZE);
		buttonsPanel.add(cancel);
		this.add(buttonsPanel);
		
		this.pack();
		this.setMinimumSize(getSize());
		this.setResizable(true);
		
		this.setLocationRelativeTo(null);
		this.setVisible(true);
		this.requestFocusInWindow();
	}
	

	/**
	 * @param panels The list the {@link ASettingsPanel}-implementations should be added to
	 * @param gui
	 */
	protected abstract void addSettingsPanels(List<ASettingsPanel> panels, SwingGUI gui);
	

	private void sizeBtn(JButton btn, Dimension size) {
		btn.setSize(size);
		btn.setMinimumSize(size);
		btn.setPreferredSize(size);
		btn.setMaximumSize(size);
	}
	
	
	private class CancelListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			onCancel();
		}
	}
	
	
	private class OkListener implements ActionListener {
		
		@Override
		public void actionPerformed(ActionEvent e) {
			onOk();
		}
	}
	
	
	/**
	 * Closes this frame and notifies the gui about it
	 */
	protected void close() {
		this.dispose();
	}
	
	
	/**
	 * Called when Cancel is clicked
	 */
	protected abstract void onCancel();
	

	/**
	 * Called when OK is clicked
	 */
	protected abstract void onOk();
}
