package edu.dhbw.mannheim.tigers.sim.gui.swing.settings;


import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.apache.commons.configuration.SubnodeConfiguration;
import org.apache.commons.configuration.tree.ConfigurationNode;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.ASimPlugin;


/**
 * This is a generic panel which displays the properties of a {@link ASimPlugin} and make it editable for the user
 * 
 * @author Gero
 * 
 */
public class PluginSettingsPanel extends ASettingsPanel {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long				serialVersionUID	= 5747877312737548308L;
	

	private final SubnodeConfiguration	config;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see PluginSettingsPanel
	 * @param title
	 * @param inputController
	 * @param config
	 */
	public PluginSettingsPanel(String title, IInputController inputController, SubnodeConfiguration config) {
		super(title, inputController);
		
		this.config = config;
	}
	

	@Override
	protected void createGui() {
		JPanel panel = createDefaultSettingsPanel("Properties");
		
		TableModel model = new PluginPropertiesTableModel(config.configurationAt("properties"));
		JTable table = new JTable(model);
		JScrollPane scrollpane = new JScrollPane(table);
		
		panel.add(scrollpane);
		add(panel);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public boolean checkInput() {
		return true;
	}
	

	@Override
	public void apply() {
		// Not used!
	}
	

	@Override
	public void revert() {
		// Not used!
	}
	

	@Override
	public void defaults() {
		// Not used!
	}
	

	@Override
	protected void loadSettings() {
		// Not used!
	}
	
	
	private class PluginPropertiesTableModel implements TableModel {
		
		private final String[]						COL_NAMES	= new String[] { "Property", "Value" };
		private final List<ConfigurationNode>	values		= new ArrayList<ConfigurationNode>();
		
		
		public PluginPropertiesTableModel(SubnodeConfiguration properties) {
			if (properties != null) {
				List<?> objs = properties.getRootNode().getChildren();
				
				for (Object obj : objs) {
					ConfigurationNode node = (ConfigurationNode) obj;
					values.add(node);
				}
			}
		}
		

		@Override
		public Class<?> getColumnClass(int columnIndex) {
			return String.class;
		}
		

		@Override
		public int getColumnCount() {
			return 2;
		}
		

		@Override
		public String getColumnName(int columnIndex) {
			return COL_NAMES[columnIndex];
		}
		

		@Override
		public int getRowCount() {
			return values.size();
		}
		

		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			ConfigurationNode node = values.get(rowIndex);
			return columnIndex == 0 ? node.getName() : node.getValue();
		}
		

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return columnIndex == 1;
		}
		

		@Override
		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			ConfigurationNode node = values.get(rowIndex);
			if (columnIndex == 1) {
				node.setValue(aValue);
			}
		}
		

		@Override
		public void removeTableModelListener(TableModelListener l) {
			// Not used
		}
		

		@Override
		public void addTableModelListener(TableModelListener l) {
			// Not used
		}
	}
}
