package edu.dhbw.mannheim.tigers.sim.util.logging;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Appender;
import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;


/**
 * Should catch all events from initialization of the log-system to the instantiation of the GUI.
 * 
 * @author Gero
 */
public class StartupLogAppender extends AppenderSkeleton {
	
	private final LinkedList<LoggingEvent>	events	= new LinkedList<LoggingEvent>();
	
	
	public StartupLogAppender() {
		super();
		
		this.setName("StartupLogAppender");
	}
	

	@Override
	protected void append(LoggingEvent arg0) {
		events.add(arg0);
	}
	

	@Override
	public void close() {
		Logger.getRootLogger().removeAppender(this);
		
		events.clear();
	}
	

	@Override
	public boolean requiresLayout() {
		return false;
	}
	

	public List<LoggingEvent> getLogEvents() {
		return new ArrayList<LoggingEvent>(events);
	}
	

	/**
	 * @param appender The {@link Appender} the saved events should be sent to
	 */
	public void pumpEventsTo(Appender appender) {
		for (LoggingEvent event : events) {
			appender.doAppend(event);
		}
	}
}
