/**
 * 
 */
package edu.dhbw.mannheim.tigers.sim.util.network;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;


/**
 * This interface declares a object that is capable of receiving something via network.
 * 
 * Together with its counterpart {@link ITransmitter} and their different implementations, it
 * represents a small, passive, flexible network framework.
 * 
 * @author Gero
 * 
 */
public interface IReceiver {
	
	public static final int	UNDEFINED_PORT	= -1;
	
	
	/**
	 * Receives a {@link DatagramPacket} from the network
	 * 
	 * @param store
	 * @return The given packet {@code store} with filled buffer (or <code>null</code> if the receiver
	 *         <code>!isReady()</code>)
	 * @throws IOException
	 */
	public DatagramPacket receive(DatagramPacket store) throws IOException;
	

	/**
	 * May throw a {@link IOException}, as implementations often call {@link DatagramSocket#close()}
	 * 
	 * @throws IOException
	 */
	public void cleanup() throws IOException;
	

	/**
	 * @return The local port the implementation is listening on
	 */
	public int getLocalPort();
	

	/**
	 * @return The address the implementation is listening on (interesting especially for multicast-groups) (
	 *         <code>null</code> if the receiver <code>!isReady()</code>)
	 */
	public InetAddress getLocalAddress();
	

	/**
	 * @return Whether the implementation is ready to receive content or not
	 */
	public boolean isReady();
}
