package edu.dhbw.mannheim.tigers.sim.util.observer;

/**
 * Implementation of the Observer-pattern
 * 
 * @author Gero
 * 
 * @param	<S>	Observable type (Subject)
 * @param	<O>	Observer type
 * @param	<E>	Event type
 */
public interface IObservable<S extends IObservable<S, O, E>, O extends IObserver<S, O, E>, E> {
	
	/**
	 * Notifies all registered observers.
	 * 
	 * @param event: The {@link Event} that changed the model
	 */
	public abstract void notifyObservers(E event);
	
	/**
	 * Notifies all registered observers. Sends the last event again!
	 */
	public abstract void notifyObservers();

	
	public abstract boolean addObservers(O... observers);
	

	public abstract boolean removeObserver(O observer);
}